package org.wso2.wsas.persistence.dao;

import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.wso2.wsas.persistence.dataobject.BindingDO;
import org.wso2.wsas.persistence.dataobject.BindingPolicyDO;
import org.wso2.wsas.persistence.dataobject.ServiceDO;
import org.wso2.wsas.persistence.dataobject.ServicePolicyDO;
import org.wso2.wsas.util.HibernateConfig;

public class BindingDAO extends BaseDAO {

	private static final Log log = LogFactory.getLog(BindingDAO.class);

	public BindingDAO(HibernateConfig hbConfig) {
		super(hbConfig);
	}

	public void updateBinding(BindingDO binding) {
		Session session = hbConfig.currentSession();
		Transaction tx = session.beginTransaction();
		try {
			binding.setLastUpdatedTime(new Date());
			session.merge(binding);
			tx.commit();
		} catch (Throwable e) {
			tx.rollback();
			String msg = "Unable to update binding";
			log.error(msg, e);
			throw new RuntimeException(msg, e);
		}
	}

	public void updatePolicy(BindingPolicyDO bindingPolicy) {
		Session session = hbConfig.currentSession();
		Transaction tx = session.beginTransaction();
		try {
			Criteria criteria = session.createCriteria(BindingPolicyDO.class,
					"policy").add(
					Restrictions.eq("policy.uuid", bindingPolicy.getUuid()));
			BindingPolicyDO bindingPolicyDO = (BindingPolicyDO) criteria
					.uniqueResult();

			if (bindingPolicyDO == null) {
				session.save(bindingPolicy);
			} else {
				bindingPolicyDO.setPolicy(bindingPolicy.getPolicy());
				session.update(bindingPolicyDO);
			}
			tx.commit();
		} catch (Throwable e) {
			tx.rollback();
			String msg = "Unable to update binding policy";
			log.error(msg, e);
			throw new RuntimeException(msg, e);
		}
	}

	public void addPolicy(String bindingName,
			BindingPolicyDO policy) {
		Session session = hbConfig.currentSession();
		Transaction tx = session.beginTransaction();
		BindingDO bindingDO;
		try {
			bindingDO = getBinding(bindingName, session);
			if (!bindingDO.getPolicies().contains(policy)) {
				bindingDO.addPolicy(policy);
				session.update(bindingDO);
			}
			tx.commit();
		} catch (Throwable e) {
			tx.rollback();
			String msg = "Unable to addPolicy";
			log.error(msg, e);
			throw new RuntimeException(msg, e);
		}
	}

	private BindingDO getBinding(String name, Session session) {
		Criteria criteria = session.createCriteria(BindingDO.class);
		criteria.add(Expression.eq("name", name.trim()));
		return (BindingDO) criteria.uniqueResult();
	}
}
