package org.wso2.wsas.persistence.dao;

import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;
import org.wso2.wsas.persistence.dataobject.BindingDO;
import org.wso2.wsas.persistence.dataobject.EndpointDO;
import org.wso2.wsas.persistence.dataobject.EndpointPolicyDO;
import org.wso2.wsas.util.HibernateConfig;

public class EndpointDAO extends BaseDAO {
	private static final Log log = LogFactory.getLog(EndpointDAO.class);

	public EndpointDAO(HibernateConfig hbConfig) {
		super(hbConfig);
	}

	public void updateEndpoint(EndpointDO endpoint) {
		Session session = hbConfig.currentSession();
		Transaction tx = session.beginTransaction();
		try {
			endpoint.setLastUpdatedTime(new Date());
			session.merge(endpoint);
			tx.commit();
		} catch (Throwable e) {
			tx.rollback();
			String msg = "Unable to update endpoint";
			log.error(msg, e);
			throw new RuntimeException(msg, e);
		}
	}

	public EndpointDO getEndpoint(String serviceId, String serviceVersion,
			String endpointName) {
		String query = "from EndpointDO as ep where ep.service.serviceIdentifierDO.serviceId='"
				+ serviceId.trim()
				+ "' and ep.service.serviceIdentifierDO.version='"
				+ serviceVersion.trim()
				+ "' and ep.name='"
				+ endpointName.trim() + ";";
		Session session = hbConfig.currentSession();
		Transaction tx = session.beginTransaction();
		EndpointDO ep;
		try {
			ep = (EndpointDO) session.createQuery(query).uniqueResult();
			tx.commit();
		} catch (Throwable e) {
			tx.rollback();
			String msg = "Cannot retrieve Endpoint";
			log.error(msg, e);
			throw new RuntimeException(msg, e);
		}
		return ep;
	}

	public EndpointDO getEndpoint(String serviceId, String serviceVersion,
			String epName, Session session) {
		String query = "from EndpointDO as ep where ep.service.serviceIdentifierDO.serviceId='"
				+ serviceId.trim()
				+ "' and ep.service.serviceIdentifierDO.version='"
				+ serviceVersion.trim()
				+ "' and ep.name='"
				+ epName.trim()
				+ ";";
		return (EndpointDO) session.createQuery(query).uniqueResult();
	}

	public BindingDO getEndpointBinding(String serviceId,
			String serviceVersion, String epName) {
		EndpointDO endpoint = getEndpoint(serviceId, serviceVersion, epName);
		if (endpoint != null) {
			return endpoint.getBinding();
		}
		return null;
	}
	
	public void updatePolicy(EndpointPolicyDO endpointPolicy) {
        Session session = hbConfig.currentSession();
        Transaction tx = session.beginTransaction();
        try {
            Criteria criteria = session.createCriteria(EndpointPolicyDO.class, "policy")
                    .add(Restrictions.eq("policy.uuid", endpointPolicy.getUuid()));
            EndpointPolicyDO endpointPolicyDO =
                    (EndpointPolicyDO) criteria.uniqueResult();

            if (endpointPolicyDO == null) {
                session.save(endpointPolicy);
            } else {
                endpointPolicyDO.setPolicy(endpointPolicy.getPolicy());
                session.update(endpointPolicyDO);
            }
            tx.commit();
        } catch (Throwable e) {
            tx.rollback();
            String msg = "Unable to update endpoint policy";
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        }
    }
}
