/*
 * Copyright 2008,2009 WSO2, Inc. http://www.wso2.org.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.mercury.modules;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisDescription;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.modules.Module;
import org.apache.axis2.util.Utils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Assertion;
import org.apache.neethi.Policy;
import org.wso2.mercury.persistence.PersistenceManager;
import org.wso2.mercury.state.RMDContext;
import org.wso2.mercury.state.RMSContext;
import org.wso2.mercury.util.MercuryConstants;
import org.wso2.mercury.util.RMDispatchInfo;
import org.wso2.mercury.workers.SequenceRemovalWorker;

/**
 * This module provides the WS-RM functionality to Axis2.
 */

public class MercuryModule implements Module {

    private static Log log = LogFactory.getLog(MercuryModule.class);

    private SequenceRemovalWorker sequenceRemovalWorker;
    // initialize the module
    public void init(ConfigurationContext configContext,
                     AxisModule module) throws AxisFault {

        // load the module parameters to the axis configuration if they are not already loaded
        AxisConfiguration axiConfiguration = configContext.getAxisConfiguration();
        if (axiConfiguration.getParameter(MercuryConstants.RMS_SEQUENCE_TIMEOUT) == null){
            axiConfiguration.addParameter(MercuryConstants.RMS_SEQUENCE_TIMEOUT,
                   Utils.getParameterValue(module.getParameter(MercuryConstants.RMS_SEQUENCE_TIMEOUT)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMS_SEQUENCE_RETRANSMIT_TIME) == null){
            axiConfiguration.addParameter(MercuryConstants.RMS_SEQUENCE_RETRANSMIT_TIME,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMS_SEQUENCE_RETRANSMIT_TIME)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMS_SEQUENCE_WORKER_SLEEP_TIME) == null){
            axiConfiguration.addParameter(MercuryConstants.RMS_SEQUENCE_WORKER_SLEEP_TIME,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMS_SEQUENCE_WORKER_SLEEP_TIME)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMS_MAXIMUM_RETRANSMIT_COUNT) == null){
            axiConfiguration.addParameter(MercuryConstants.RMS_MAXIMUM_RETRANSMIT_COUNT,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMS_MAXIMUM_RETRANSMIT_COUNT)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMD_SEQUENCE_TIMEOUT) == null){
            axiConfiguration.addParameter(MercuryConstants.RMD_SEQUENCE_TIMEOUT,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMD_SEQUENCE_TIMEOUT)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMD_SEQUENCE_RETRANSMIT_TIME) == null){
            axiConfiguration.addParameter(MercuryConstants.RMD_SEQUENCE_RETRANSMIT_TIME,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMD_SEQUENCE_RETRANSMIT_TIME)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.RMD_SEQUENCE_WORKER_SLEEP_TIME) == null){
            axiConfiguration.addParameter(MercuryConstants.RMD_SEQUENCE_WORKER_SLEEP_TIME,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.RMD_SEQUENCE_WORKER_SLEEP_TIME)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.INVOKER_TIMEOUT) == null){
            axiConfiguration.addParameter(MercuryConstants.INVOKER_TIMEOUT,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.INVOKER_TIMEOUT)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.INVOKER_WORKER_SLEEP_TIME) == null){
            axiConfiguration.addParameter(MercuryConstants.INVOKER_WORKER_SLEEP_TIME,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.INVOKER_WORKER_SLEEP_TIME)));
        }
        if (axiConfiguration.getParameter(MercuryConstants.ENFORCE_RM) == null){
            axiConfiguration.addParameter(MercuryConstants.ENFORCE_RM,
                    Utils.getParameterValue(module.getParameter(MercuryConstants.ENFORCE_RM)));
        }

        long sequenceRemvalWorkerSleepTime = 30000;
        if (axiConfiguration.getParameter(MercuryConstants.SEQUENCE_REMOVAL_WORKER_SLEEP_TIME) != null) {
            sequenceRemvalWorkerSleepTime =
                    Long.parseLong((String) axiConfiguration.getParameter(
                            MercuryConstants.SEQUENCE_REMOVAL_WORKER_SLEEP_TIME).getValue());
        } else if (module.getParameter(MercuryConstants.SEQUENCE_REMOVAL_WORKER_SLEEP_TIME) != null) {
            sequenceRemvalWorkerSleepTime =
                    Long.parseLong((String) module.getParameter(MercuryConstants.SEQUENCE_REMOVAL_WORKER_SLEEP_TIME).getValue());
        }

        // register two new context objects
        configContext.setProperty(MercuryConstants.RMS_CONTEXT, new RMSContext(configContext));
        configContext.setProperty(MercuryConstants.RMD_CONTEXT, new RMDContext(configContext));
        configContext.setProperty(MercuryConstants.RM_DISPATCH_INFO, new RMDispatchInfo());

        //initialize the persistence storage manager and stored save it in
        // configuratin context
        String persistanceManagerClass = null;
        if (axiConfiguration.getParameter(MercuryConstants.RM_PERSISTANCE_MANAGER) != null){
            persistanceManagerClass = (String) axiConfiguration.getParameter(
                    MercuryConstants.RM_PERSISTANCE_MANAGER).getValue();
        } else if (module.getParameter(MercuryConstants.RM_PERSISTANCE_MANAGER) != null) {
            persistanceManagerClass = (String) module.getParameter(MercuryConstants.RM_PERSISTANCE_MANAGER).getValue();
        }
        if (persistanceManagerClass != null) {
            try {
                PersistenceManager persistenceManager =
                        (PersistenceManager) Class.forName(persistanceManagerClass).newInstance();
                configContext.setProperty(MercuryConstants.RM_PERSISTANCE_MANAGER, persistenceManager);
            } catch (InstantiationException e) {
                log.error("Can not instantiate persistence manager", e);
                throw new AxisFault("Can not instantiate persistence manager", e);
            } catch (IllegalAccessException e) {
                log.error("Illegal Access to persistence manager", e);
                throw new AxisFault("Illegal Access to persistence manager", e);
            } catch (ClassNotFoundException e) {
                log.error("Persistance manager class not found", e);
                throw new AxisFault("Persistance manager class not found", e);
            }
        }

        // start the sequence remover.
        sequenceRemovalWorker = new SequenceRemovalWorker(configContext);
        sequenceRemovalWorker.setSequenceRemovalWorkerSleepTime(sequenceRemvalWorkerSleepTime);
        configContext.getThreadPool().execute(sequenceRemovalWorker);

    }

    public void engageNotify(AxisDescription axisDescription) throws AxisFault {
        //TODO: implement if necessary
    }

    public void shutdown(ConfigurationContext configurationContext) throws AxisFault {
        // remove context objects
        log.info("Shutting down Mecury");
        sequenceRemovalWorker.shutDownWorker();
        configurationContext.removeProperty(MercuryConstants.RMS_CONTEXT);
        configurationContext.removeProperty(MercuryConstants.RMD_CONTEXT);
        configurationContext.removeProperty(MercuryConstants.RM_DISPATCH_INFO);
    }

    public void applyPolicy(Policy policy, AxisDescription axisDescription) throws AxisFault {
        //TODO: implement if necessary
    }

    public boolean canSupportAssertion(Assertion assertion) {
        //TODO: implement if necessary
        return false;
    }
}
