package org.wso2.ws.dataservice.test;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.util.XMLPrettyPrinter;

public class MySQLDataSourceTest extends DataServiceBaseTestCase {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Create the test case
	 *
	 * @param testName name of the test case
	 */
	public MySQLDataSourceTest(String testName) {
		super(testName);
	}

	/**
	 * @doc : exposes single mysql table as a service,
	 * no parameters.Generates a response based on a 
	 * select query
	 */
	public void testSingleTableNoParams() {
		String endPointUrl = epr + "MySQLService";
		System.out.println("###########################################");
		System.out.println("Testing service : "+endPointUrl);
		System.out.println("###########################################");
		EndpointReference targetEPR = new EndpointReference(endPointUrl);
		try {
			OMElement payload = getSelectPayload();
			Options options = new Options();
			options.setTo(targetEPR);
			options.setAction("urn:results1");
			ServiceClient sender = new ServiceClient();
			sender.setOptions(options);
			OMElement result = sender.sendReceive(payload);
			XMLPrettyPrinter.prettify(result, System.out);
			assertNotNull(result);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		assertTrue(true);
	}

	/**
	 * 
	 * @doc : payload for SELECT service call
	 */
	private static OMElement getSelectPayload() {
		OMFactory fac = OMAbstractFactory.getOMFactory();
		OMNamespace omNs = fac.createOMNamespace(
				"http://example1.org/example1", "example1");
		OMElement method = fac.createOMElement("results1", omNs);
		return method;
	}

	/**
	 * @doc : Inserts a record to mysql table.The operation
	 * on database does not return any value. Objective of this
	 * test is to check the behavior when there is not response.
	 * (Result element should not be defined on Query element) 
	 */
	public void testInsertNoReturn() {
		String endPointUrl = epr + "MySQLService";
		System.out.println("###########################################");
		System.out.println("Testing service : "+endPointUrl);
		System.out.println("###########################################");
		EndpointReference targetEPR = new EndpointReference(endPointUrl);
		try {
			OMElement payload = getInsertPayload();
			Options options = new Options();
			options.setTo(targetEPR);
			options.setAction("urn:insert");
			long timeOut = 3 * 60 * 1000;
			options.setTimeOutInMilliSeconds(timeOut);
			
			ServiceClient sender = new ServiceClient();
			sender.setOptions(options);
			sender.sendReceive(payload);
			//OMElement result = sender.sendReceive(payload);
			//XMLPrettyPrinter.prettify(result, System.out);
			//assertNotNull(result);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		assertTrue(true);
	}

	/**
	 * 
	 * @doc : payload for INSERT service call
	 */
	private static OMElement getInsertPayload() {
		OMFactory fac = OMAbstractFactory.getOMFactory();
		OMNamespace omNs = fac.createOMNamespace(
				"http://example1.org/example1", "example1");
		OMElement method = fac.createOMElement("insert", omNs);
		
		OMElement param1 = fac.createOMElement("id", omNs);
		param1.setText("5");
		OMElement param2 = fac.createOMElement("name", omNs);
		param2.setText("Dutugemunu");
		OMElement param3 = fac.createOMElement("deptId", omNs);
		param3.setText("1");
		OMElement param4 = fac.createOMElement("address", omNs);
		param4.setText("Anuradhapura");
		OMElement param5 = fac.createOMElement("email", omNs);
		param5.setText("dutu@gemunu.org");
		
		method.addChild(param1);
		method.addChild(param2);
		method.addChild(param3);
		method.addChild(param4);
		method.addChild(param5);
		return method;
	}
	
}
