 /*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.context;

import org.eclipse.wst.command.internal.env.context.PersistentContext;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import java.util.List;
import java.lang.reflect.*;
/**
 * Context that persist the Data Service as Eclipse Persistence Context
 */
public class PersistentDSContext extends PersistentContext implements DSContext {

	//Static reference of the context
	private static PersistentDSContext dsContext = null;
	private List queryData;
	private List operationData = null;
	private Method method = null;
	private Object belongingObject;
	private String serviceName;
	
	//Private PersistentDSContext Constructor
	private PersistentDSContext() {
		super(WSO2DataservicePlugin.getDefault());
	}

	//Singleton
	public static PersistentDSContext getInstance() {
		if (dsContext == null) {
			dsContext = new PersistentDSContext();
			dsContext.load();
		}
		return dsContext;
	}

	//Load the defaults
	public void load() {
		setDefault(PREFERENCE_DS_CONFIG, DSPreferencesDefaults.getDSConfiguration());
		setValue(PREFERENCE_DS_CONFIG, DSPreferencesDefaults.getDSConfiguration());
		setDefault(PREFERENCE_CARBON_HOME, DSPreferencesDefaults.getCarbonHome());
		setValue(PREFERENCE_CARBON_HOME, DSPreferencesDefaults.getCarbonHome());
	}

	/**
	 * receive the data service configuration as String from context
	 */
	public String getDSConfig() {
		return getValueAsString(PREFERENCE_DS_CONFIG);
	}

	/**
	 * set the data service configuration in to the context
	 */
	public void setDSConfig(String dsConfig) {
		setValue(PREFERENCE_DS_CONFIG, dsConfig);
	}
	
	/**
	 * receive the data service configuration as String from context
	 */
	public String getCarbonHome() {
		return getValueAsString(PREFERENCE_CARBON_HOME);
	}

	/**
	 * set the data service configuration in to the context
	 */
	public void setCarbonHome(String dsConfig) {
		setValue(PREFERENCE_CARBON_HOME, dsConfig);
	}
	
	
	public List getQueryData(){
		return queryData;
	}
	
	public void setQueryData(List queryData){
		this.queryData=queryData;
	}
	
	public List getOperationData(){
		return operationData;
	}
	
	public void setOperationData(List operationData){
		this.operationData=operationData;
	}
	
	public void updateOperationDataOnXml(){
		if (method != null){
			try{
				method.invoke(belongingObject,null);
		    } catch (SecurityException e) {
		    	e.printStackTrace(); 
		    } catch (IllegalAccessException e) {
		        e.printStackTrace(); 
		    } catch (IllegalArgumentException e) {
		        e.printStackTrace(); 
		    } catch (InvocationTargetException e) {
		        e.printStackTrace(); 
		    }

		}
	}	
	
	public void setMethodForUpdateOperationDataOnXml(Method method, Object methodOwningInstance){
		this.method=method;
		this.belongingObject=methodOwningInstance;
	}

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
        this.serviceName = serviceName;		
	}
}
