/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.util.FontUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepThree;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepTwo;
import org.wso2.ws.dataservice.ide.context.*;

import java.util.List;
import java.util.ArrayList;

/**
 * Dialog represent the Add Input Parameter configuration UI
 */
public class AddInputParamDialog extends Dialog{

	private Combo sqlTypeCombo;
	private Combo inOutTypeCombo;
	private List mappingData;
	
	private Text nameText;
	private Text ordinalText;
	
	private boolean isEditMode=false;
	private List tmpData;
	
	public AddInputParamDialog(Shell parentShell, List data) {
		super(parentShell);
		isEditMode=(data!=null);
		tmpData=data;	
		mappingData=new ArrayList();	
		resetMappingData();
	}

	private void resetMappingData(){
		mappingData.clear();
		mappingData.add("");
		mappingData.add("");
		mappingData.add("");
		mappingData.add("");
	}
	

	/**
	 * Create the dialog area with the controls
	 */
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		FontUtil fontUtil = FontUtil.getInstance(container.getDisplay());

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;

		Label label = new Label(container, SWT.NULL);
		label.setText("Add New Input Param");
		label.setFont(fontUtil.getArealBoldFont());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("Name *");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		nameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		nameText.setLayoutData(gd);
		nameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				pathChanged();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("SQL Type *");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		sqlTypeCombo = new Combo(container,SWT.READ_ONLY);
		loadSQLTypeCombo();
		sqlTypeCombo.select(0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		sqlTypeCombo.setLayoutData(gd);
		sqlTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				comboChanged();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("In/Out Type **");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		inOutTypeCombo = new Combo(container,SWT.READ_ONLY);
		loadInOutTypeCombo();
		inOutTypeCombo.select(0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		inOutTypeCombo.setLayoutData(gd);
		inOutTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				comboChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Ordinal **");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		ordinalText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		ordinalText.setLayoutData(gd);
		ordinalText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				pathChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("** Required only for stored procedures");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		comboChanged();
		if (isEditMode){
			updateMappingData(tmpData);
			nameText.setEditable(false);
			mappingData=tmpData;
		}
		return super.createDialogArea(parent);
	}

	private void updateMappingData(List data){
		int i=sqlTypeCombo.indexOf(
				data.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_SQL_TYPE).toString());
		if (i>=0) sqlTypeCombo.select(i);
		
		i=inOutTypeCombo.indexOf(
				data.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_INOUT_TYPE).toString());
		if (i>=0) inOutTypeCombo.select(i);

		nameText.setText(data.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString());
		ordinalText.setText(data.get
				(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_ORDINAL).toString());
	}
	
	private void pathChanged() {
		mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME, 
				nameText.getText().trim());
		mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_ORDINAL, 
				ordinalText.getText().trim());
		//update the data model
	}

	/**
	 * Add the content to the sql type combo.
	 */
	private void loadSQLTypeCombo() {
		sqlTypeCombo.add("STRING");
		sqlTypeCombo.add("INTEGER");
		sqlTypeCombo.add("DOUBLE");
		sqlTypeCombo.add("DATE");
		sqlTypeCombo.add("TIME");
		sqlTypeCombo.add("TIMESTAMP");
	}

	/**
	 * Add the content to the in/out type combo.
	 */
	private void loadInOutTypeCombo() {
		inOutTypeCombo.add("IN");
		inOutTypeCombo.add("OUT");
		inOutTypeCombo.add("IN/OUT");
	}

	/**
	 * Handle the Combo Changed scenario
	 */
	private void comboChanged() {
		if (sqlTypeCombo.getSelectionIndex()>-1) 
			mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_SQL_TYPE, 
					sqlTypeCombo.getItem(sqlTypeCombo.getSelectionIndex()).trim());
		if (inOutTypeCombo.getSelectionIndex()>-1) 
			mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_INOUT_TYPE, 
					inOutTypeCombo.getItem(inOutTypeCombo.getSelectionIndex()).trim());
		//Combo updated
	}

	public List getMappingData(){
		return mappingData;
	}
	
	protected void okPressed(){
		if (isAllDataValid()){
			super.okPressed();
		}
	}
	
	private boolean isAllDataValid(){
		String validateString;
		validateString=mappingData.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString();
		if (validateString.equals("")){
			showMsg("Name cannot be empty");
			return false;
		}else{
			if (!WSO2DataserviceWizardStepTwo.isStringValidXmlTag(validateString)){
				showMsg("Name string '"+validateString+"' is not a valid xml tag string.");
				return false;
			}
		}
		validateString=mappingData.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_ORDINAL).toString();
		if (!validateString.equals("")){
			try{
				Integer.parseInt(validateString);
			}catch(Exception e){
				showMsg("Ordinal needs to be a numerical value");
				return false;
			}
		}

		return true;
	}
	
	private void showMsg(String msg){
		MessageBox messageBox =new MessageBox(getShell(),SWT.OK);
		 messageBox.setMessage(msg);
		 messageBox.open();
	}
	
}


