/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;
import org.wso2.ws.dataservice.ide.util.FontUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepThree;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepTwo;

public class AddOperationDialog extends Dialog{

	private Combo queryTypeCombo;
	private String dialogHeadingPrefix;
	private Text operationNameText;
	private Table tableInputMaps;
	private Button editButtonInputMap;
	private Label labelMessage;
	
	private List tmpOIDList;
	private PersistentDSContext dsContext;
	private WSO2DataserviceWizardStepThree parentPage;
	private List configList;
	private HashMap valueMaps;

	private boolean isEditMode = false;
	
	/**
	 * Dialog represent the Add Operations configuration UI
	 */
	public AddOperationDialog(Shell parentShell, String headingPrefix, WSO2DataserviceWizardStepThree page) {
		super(parentShell);
		dialogHeadingPrefix = headingPrefix;
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		parentPage = page;
		configList = new ArrayList();
		valueMaps = new HashMap();
		resetConfigData();
		isEditMode = headingPrefix.equals(WSO2DataserviceWizardConstant.editTag);
	}
	
	/**
	 * clear the query data
	 */
	private void resetConfigData(){
		configList.clear();
		configList.add("");
		configList.add("");
		configList.add(valueMaps);
	}

	/**
	 * Create the dialog area with the controls
	 */
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		FontUtil fontUtil = FontUtil.getInstance(container.getDisplay());
        
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;
		
		Label label = new Label(container, SWT.BOLD);
		label.setFont(fontUtil.getArealBoldFont());
		label.setText(dialogHeadingPrefix + " Operation Configuration");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.NULL);
		label.setText("Operation Name");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		operationNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		operationNameText.setLayoutData(gd);
		operationNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				operationNameChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		labelMessage = new Label(container, SWT.NULL);
		labelMessage.setForeground(new org.eclipse.swt.graphics.Color(getShell().getDisplay(),255,0,0));
		gd = new GridData();
		gd.horizontalSpan = 2;
		label.setLayoutData(gd);
		labelMessage.setText("                                             ");
		
		label = new Label(container, SWT.NULL);
		label.setText("Query");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		queryTypeCombo = new Combo(container,SWT.NULL);
		loadQueryCombo();
		queryTypeCombo.select(0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		queryTypeCombo.setLayoutData(gd);
		queryTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});
		queryTypeCombo.setEnabled(false);
		
		label = new Label(container, SWT.BOLD);
		label.setFont(fontUtil.getArealBoldFont());
		label.setText("Operation Parameter Mappings");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.NULL);
		label.setText("Operation");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		tableInputMaps = new Table(container,
				SWT.SINGLE|SWT.FULL_SELECTION|SWT.V_SCROLL|SWT.H_SCROLL);
		tableInputMaps.setLinesVisible(true);
		tableInputMaps.setHeaderVisible(true); 
		tableInputMaps.setLayoutData(gd);
		declareColumn(tableInputMaps,100,"Name");
		declareColumn(tableInputMaps,75,"Value");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		gd.heightHint = 100;
		tableInputMaps.setLayoutData(gd);
		tableInputMaps.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				handleInputMapTableItemSelected();
			}
		});
		tableInputMaps.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e){
				handleInputMapEditQuery();
			}
		});
		tableInputMaps.setEnabled(false);
		
		tableInputMaps.redraw();
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		label.setLayoutData(gd);
		editButtonInputMap = new Button(container, SWT.PUSH);
		editButtonInputMap.setText("Edit Selected value");
		editButtonInputMap.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleInputMapEditQuery();
			}
		});
		editButtonInputMap.setEnabled(false);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		editButtonInputMap.setLayoutData(gd);

		createListFromQuerySConfig();
		if (isEditMode){
			String oId = parentPage.getSelectedOperationId();
			List tmpConfigList = parentPage.getOperationData(oId);
			resetOperationData(tmpConfigList);
			configList = tmpConfigList;
			operationNameText.setEditable(false);
		}
		return super.createDialogArea(parent);
	}

	/**
	 * sets the UI component values to the values in a List which has query information  
	 * @param tmpConfigList: List containing the query information 
	 */
	private void resetOperationData(List tmpConfigList){
		if (tmpConfigList!= null){
			operationNameText.setText(tmpConfigList.get(WSO2DataserviceWizardStepThree.SQLOP_OPERATIONID).toString());
			if (queryTypeCombo.indexOf(tmpConfigList.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString()) > -1)
				queryTypeCombo.select(queryTypeCombo.indexOf(tmpConfigList.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString()));
			for (int i = 0; i < tableInputMaps.getItemCount(); i++) {
				String name=tableInputMaps.getItem(i).getText();
				tableInputMaps.getItem(i).setText(new String[]{name, ((HashMap)tmpConfigList.get(WSO2DataserviceWizardStepThree.SQLOP_VALUE_MAPS)).get(name).toString()});
			}
			valueMaps=(HashMap)tmpConfigList.get(WSO2DataserviceWizardStepThree.SQLOP_VALUE_MAPS);
		} 	
	}

	private void declareColumn(Table table, int width,String colName){
		TableColumn column = new TableColumn(table,SWT.NONE);
		column.setWidth(width);
		column.setText(colName);
	}
	
	/**
	 * Handle when an item in the query table is selected 
	 */
	private void handleInputMapTableItemSelected(){
		editButtonInputMap.setEnabled(tableInputMaps.getSelectionIndex()!= -1);
	}
	
	/**
	 * Handle edit Query
	 */
	private void handleInputMapEditQuery() {
		if (tableInputMaps.getSelectionIndex() >= 0){
			String name = tableInputMaps.getItem(tableInputMaps.getSelectionIndex()).getText();
			InputBox input=new InputBox(getShell());
			input.setMessage("Enter a value for the parameter '"+name+"'.");
			input.setValue((String)valueMaps.get(name));
			if (input.open() == org.eclipse.jface.window.Window.OK){
				String result = input.getTypedValue();
				valueMaps.put(name, result);
				tableInputMaps.getItem(tableInputMaps.getSelectionIndex()).setText(new String[]{name, result});
			}
		}
	}

	
	/**
	 * Handle the Operation Name Changed
	 */
	private void operationNameChanged() {
		configList.set(WSO2DataserviceWizardStepThree.SQLOP_OPERATIONID, operationNameText.getText());
		if (isQueryIDAlreadyPresent(
				configList.get(WSO2DataserviceWizardStepThree.SQLOP_OPERATIONID).toString())){
			queryTypeCombo.setEnabled(isEditMode);
			tableInputMaps.setEnabled(isEditMode);
			if (isEditMode) labelMessage.setText(""); 
			else labelMessage.setText("Operation ID already exists.");
		}else{
			labelMessage.setText("");
			queryTypeCombo.setEnabled(!configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString().equals(""));
			tableInputMaps.setEnabled(!configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString().equals(""));
		}
		//update the data model
	}
	
	/**
	 * Add the content to the query combo
	 */
	private void loadQueryCombo() {
		//Load the query details
		List queryIds=WSO2DataserviceWizardStepTwo.getQueryIdList(dsContext.getQueryData());
		for (int i = 0; i < queryIds.size(); i++) {
			queryTypeCombo.add((String)queryIds.get(i));	
		}
	}
	
	/**
	 * Handle the Cancel button event 
	 */
	protected void cancelPressed() {
		super.cancelPressed();
		//Remove the DS Config from the 
	}

	/**
	 * Handle the OK button pressed
	 */
	protected void okPressed() {
		super.okPressed();
		//Update the DS Configuration
		parentPage.setOperationData(configList.get(WSO2DataserviceWizardStepThree.SQLOP_OPERATIONID).toString(), configList);
		parentPage.configChanged();
	}
	
	
	/**
	 * This method updates the List form current RDBMS Configuration.
	 * @return RDBMS COnfiguration 
	 */
	private void createListFromQuerySConfig(){
		configList.set(WSO2DataserviceWizardStepThree.SQLOP_QUERYID,queryTypeCombo.getItem(queryTypeCombo.getSelectionIndex()));
		fillTable(configList.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString());
		//configList.add(paramText.getText());
	}
	
	/**
	 * checks whether the given query id is already present in the query list 
	 * @param qid: query id tested
	 * @return true if the id is already present
	 */
	private boolean isQueryIDAlreadyPresent(String operationId){
		if (operationId.equals("")) return false;
		if (tmpOIDList == null) tmpOIDList = parentPage.getOperationIdList();
		return (tmpOIDList.indexOf(operationId) >= 0);
	}
	
	private void fillTable(String queryId){
		List mappings=parentPage.getQueryDataInputMappingList(queryId, dsContext.getQueryData());
		tableInputMaps.removeAll();
		valueMaps.clear();
		for (int i = 0; i < mappings.size(); i++) {
			List mapping=(List)mappings.get(i);
			TableItem items = new TableItem(tableInputMaps, SWT.NONE);
			items.setText(mapping.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString());
			valueMaps.put(mapping.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString(), "");
		}
		tableInputMaps.redraw();
	}
	/**
	 * Get the Configuration List
	 * @return
	 */
	public List getOperationList() {
		return configList;
	}

	class InputBox extends Dialog{
		
		private Text nameText;
		private String value;
		private String msg;
		private Label lblMessage;
		
		public InputBox(Shell parentShell) {
			super(parentShell);
		}

		public void setMessage(String message){
			msg=message;
		}

		public void setValue(String value){
			this.value=value;
		}
		
		
		/**
		 * Create the dialog area with the controls
		 */
		protected Control createDialogArea(final Composite parent) {
			Composite container = (Composite) super.createDialogArea(parent);
			FontUtil fontUtil = FontUtil.getInstance(container.getDisplay());

			GridLayout layout = new GridLayout();
			container.setLayout(layout);
			layout.numColumns = 3;
			layout.verticalSpacing = 10;

			lblMessage = new Label(container, SWT.NULL);
			lblMessage.setFont(fontUtil.getArealBoldFont());
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalSpan = 3;
			lblMessage.setLayoutData(gd);
			lblMessage.setText(msg);

			nameText = new Text(container, SWT.BORDER | SWT.SINGLE);
			gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalSpan = 3;
			nameText.setLayoutData(gd);
			nameText.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					pathChanged();
				}
			});
			nameText.setText(this.value);
			return super.createDialogArea(parent);
		}
		
		private void pathChanged() {
			value=nameText.getText();
		}

		public String getTypedValue(){
			return value;
		}
		
		protected void okPressed(){
			if (isAllDataValid()){
				super.okPressed();
			}
		}
		
		private boolean isAllDataValid(){
			return true;
		}
		
	}


}