/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.util.FontUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepTwo;

/**
 * Dialog represent the Add Output Mapping configuration UI
 */
public class AddOutputMappingDialog extends Dialog{

	private Combo mappingTypeCombo;
	private List mappingData;
	
	private Text sqlColumnNameText;
	private Text outputFieldNameText;

	private boolean isEditMode = false;
	private List tmpData;
	
	public AddOutputMappingDialog(Shell parentShell, List data) {
		super(parentShell);
		isEditMode = (data !=  null);
		tmpData = data;	
		mappingData = new ArrayList();
		resetMappingData();
	}

	/**
	 * clear the values in the list which keeps the mapping data
	 */
	private void resetMappingData(){
		mappingData.clear();
		mappingData.add("");
		mappingData.add("");
		mappingData.add("");
		mappingData.add("");
	}

	/**
	 * Create the dialog area with the controls
	 */
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		FontUtil fontUtil = FontUtil.getInstance(container.getDisplay());

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;

		Label label = new Label(container, SWT.NULL);
		label.setText("Add New Output Mapping");
		label.setFont(fontUtil.getArealBoldFont());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("Mapping Type");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		mappingTypeCombo = new Combo(container,SWT.READ_ONLY);
		loadMappingTypeCombo();
		mappingTypeCombo.select(0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		mappingTypeCombo.setLayoutData(gd);
		mappingTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				comboChanged();
			}
		});

		
		label = new Label(container, SWT.NULL);
		label.setText("Output Field Name");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		outputFieldNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		outputFieldNameText.setLayoutData(gd);
		outputFieldNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				pathChanged();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("SQL Column Name");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		sqlColumnNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		sqlColumnNameText.setLayoutData(gd);
		sqlColumnNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				pathChanged();
			}
		});
		comboChanged();
		if (isEditMode){
			updateMappingData(tmpData);
			mappingData = tmpData;
		}
		return super.createDialogArea(parent);
	}

	/**
	 * Add the content to the mapping type combo.
	 */
	private void loadMappingTypeCombo() {
		mappingTypeCombo.add("ELEMENT");
		mappingTypeCombo.add("QUERY");
		mappingTypeCombo.add("ATTRIBUTE");
	}

	/**
	 * when the text box values change
	 */
	private void pathChanged() {
		mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_SQL_COL,
				sqlColumnNameText.getText().trim());
		mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_OUT_FIELD,
				outputFieldNameText.getText().trim());
		//update the data model
	}
	
	/**
	 * Ensures that both combobox fields are set.
	 */
	private void comboChanged() {
		if (mappingTypeCombo.getSelectionIndex() > -1) 
			mappingData.set(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_TYPE, 
					mappingTypeCombo.getItem(mappingTypeCombo.getSelectionIndex()).trim());
	}
	
	public List getMappingData(){
		return mappingData;
	}
	
	/**
	 * update the UI components with the values in a list which contains mapping data
	 * @param data: list which contains mapping data
	 */
	private void updateMappingData(List data){
		int i = mappingTypeCombo.indexOf(
				data.get(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_TYPE).toString());
		if (i >= 0) mappingTypeCombo.select(i);
		
		sqlColumnNameText.setText(data.get(
				WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_SQL_COL).toString());
		outputFieldNameText.setText(data.get(
				WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_OUT_FIELD).toString());
	}
	
	protected void okPressed(){
		if (isAllDataValid()){
			super.okPressed();
		}
	}
	
	/**
	 * validate the entered data
	 * @return true if the entered data are all valid
	 */
	private boolean isAllDataValid(){
		String validateString;
		validateString = mappingData.get(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_OUT_FIELD).toString();
		if (validateString.equals("")){
			showMsg("Output field name cannot be empty.");
			return false;
		}else{
			if (!WSO2DataserviceWizardStepTwo.isStringValidXmlTag(validateString)){
				showMsg("Output field name string '"+validateString+"' is not a valid xml tag string.");
				return false;
			}
		}
		
		validateString = mappingData.get(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_SQL_COL).toString();
		if (validateString.equals("")){
			showMsg("SQL column name cannot be empty.");
			return false;
		}

		return true;
	}
	
	/**
	 * show a message box
	 * @param message
	 */
	private void showMsg(String message){
		MessageBox messageBox  = new MessageBox(getShell(),SWT.OK);
		 messageBox.setMessage(message);
		 messageBox.open();
	}
	
	
}