/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.DSContext;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.util.FontUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepThree;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepTwo;

/**
 * Dialog represent the Add Query configuration UI
 */
public class AddQueryDialog extends Dialog{

	private Button inputMappingButton;
	private Button outMappingButton;
	private String dialogHeadingPrefix;
	private Text queryIDText;
	private Label labelMessage;
	
	private StyledText sqlStatementText;
	private Text groupByElementText;
	private Text rowNameText;
	private Text rowNamespaceText;
	private Table tableInputMaps;
	private Button editButtonInputMap;
	private Button deleteButtonInputMap;
	
	private Table tableOutputMaps;
	private Button editButtonOutputMap;
	private Button deleteButtonOutputMap;
	
	private WSO2DataserviceWizardStepTwo parentPage;
	private List configList;
	private List inputMapList;
	private List outputMapList;
	private List tmpQIDList;
	
	private boolean isEditMode = false;
	
	public AddQueryDialog(Shell parentShell, String headingPrefix,
			WSO2DataserviceWizardStepTwo page) {
		super(parentShell);
		configList = new ArrayList();
		inputMapList = new ArrayList();
		outputMapList = new ArrayList();
		resetConfigData();
		dialogHeadingPrefix = headingPrefix;
		isEditMode = headingPrefix.equals(WSO2DataserviceWizardConstant.editTag);
		//Obtain the current context from eclipse preferences.

		parentPage = page;
	}

	/**
	 * clear the query data
	 */
	private void resetConfigData(){
		configList.clear();
		configList.add("");
		configList.add("");
		configList.add("");
		configList.add("");
		configList.add("");
		configList.add(inputMapList);
		configList.add(outputMapList);
	}
	/**
	 * Create the dialog area with the controls
	 */
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		FontUtil fontUtil = FontUtil.getInstance(container.getDisplay());

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;

		Label label = new Label(container, SWT.BOLD);
		label.setFont(fontUtil.getArealBoldFont());
		label.setText(dialogHeadingPrefix + " SQL Query / Stored Procedure Configuration");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("Query ID *");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		queryIDText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		queryIDText.setLayoutData(gd);
		queryIDText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		label.setLayoutData(gd);

		labelMessage = new Label(container, SWT.NULL);
		labelMessage.setForeground(new org.eclipse.swt.graphics.Color(getShell().getDisplay(),255,0,0));
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		labelMessage.setText("                                             ");

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setFont(fontUtil.getArealBoldFont());
		label.setText("SQL");
		gd = new GridData();
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("SQL Statement *");
		gd = new GridData();
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		sqlStatementText = new StyledText(container, 
				SWT.MULTI | SWT.WRAP | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		gd = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
		gd.horizontalSpan = 3;
		gd.verticalSpan = 10;
		sqlStatementText.setLayoutData(gd);
		sqlStatementText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});

		tableInputMaps = new Table(container,
				SWT.SINGLE|SWT.FULL_SELECTION|SWT.V_SCROLL|SWT.H_SCROLL);
		tableInputMaps.setLinesVisible(true);
		tableInputMaps.setHeaderVisible(true); 
		tableInputMaps.setLayoutData(gd);
		declareColumn(tableInputMaps,100,"Name");
		declareColumn(tableInputMaps,75,"SQL Type");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		gd.heightHint = 100;
		tableInputMaps.setLayoutData(gd);
		tableInputMaps.setVisible(false);
		tableInputMaps.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				handleInputMapTableItemSelected();
			}
		});
		tableInputMaps.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e){
				handleInputMapEditQuery();
			}
		});
		tableInputMaps.redraw();
		tableInputMaps.setVisible(true);
		setTableVisibleStatus(tableInputMaps, false);
		
		inputMappingButton = new Button(container, SWT.NULL);
		inputMappingButton.setText("Add New Input Mapping");
		gd = new GridData();
		gd.horizontalSpan = 1;
		inputMappingButton.setLayoutData(gd);
		inputMappingButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleNewInputMapping();
			}
		});
		inputMappingButton.setEnabled(false);
		
		editButtonInputMap = new Button(container, SWT.PUSH);
		editButtonInputMap.setText("Edit Selected Mapping");
		editButtonInputMap.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleInputMapEditQuery();
			}
		});
		editButtonInputMap.setEnabled(false);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		editButtonInputMap.setLayoutData(gd);

		deleteButtonInputMap =  new Button(container, SWT.PUSH);
		deleteButtonInputMap.setText("Delete Selected Mapping");
		deleteButtonInputMap.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleInputMapDeleteQuery();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		deleteButtonInputMap.setLayoutData(gd);
		deleteButtonInputMap.setEnabled(false);		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.BOLD);
		label.setFont(fontUtil.getArealBoldFont());
		label.setText(" Result ( Output Mapping ) ");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("Group By Element");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		groupByElementText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		groupByElementText.setLayoutData(gd);
		groupByElementText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("Row Name");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		rowNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		rowNameText.setLayoutData(gd);
		rowNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("Row Namespace");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);

		rowNamespaceText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		rowNamespaceText.setLayoutData(gd);
		rowNamespaceText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromQuerySConfig();
			}
		});

		tableOutputMaps = new Table(container,
				SWT.SINGLE|SWT.FULL_SELECTION|SWT.V_SCROLL|SWT.H_SCROLL);
		tableOutputMaps.setLinesVisible(true);
		tableOutputMaps.setHeaderVisible(true); 
		tableOutputMaps.setLayoutData(gd);
		declareColumn(tableOutputMaps,100,"Mapping type");
		declareColumn(tableOutputMaps,100,"Output Field Name");
		declareColumn(tableOutputMaps,100,"Result column");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		gd.heightHint = 100;
		tableOutputMaps.setLayoutData(gd);
		tableOutputMaps.setVisible(false);
		tableOutputMaps.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				handleOutputMapTableItemSelected();
			}
		});
		tableOutputMaps.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e){
				handleOutputMapEditQuery();
			}
		});
		tableOutputMaps.redraw();
		tableOutputMaps.setVisible(true);
		setTableVisibleStatus(tableOutputMaps, false);
		
		outMappingButton = new Button(container, SWT.NULL);
		outMappingButton.setText("Add New Output Mapping");
		gd = new GridData();
		gd.horizontalSpan = 1;
		outMappingButton.setLayoutData(gd);
		outMappingButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleNewOutputMapping();
			}
		});
		outMappingButton.setEnabled(false);

		editButtonOutputMap = new Button(container, SWT.PUSH);
		editButtonOutputMap.setText("Edit Selected Mapping");
		editButtonOutputMap.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleOutputMapEditQuery();
			}
		});
		editButtonOutputMap.setEnabled(false);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		editButtonOutputMap.setLayoutData(gd);

		deleteButtonOutputMap =  new Button(container, SWT.PUSH);
		deleteButtonOutputMap.setText("Delete Selected Mapping");
		deleteButtonOutputMap.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleOutputMapDeleteQuery();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		deleteButtonOutputMap.setLayoutData(gd);
		deleteButtonOutputMap.setEnabled(false);		
		
		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		/*
		 * setup the environment for edit mode
		 */
		if (isEditMode){
			String qId = parentPage.getSelectedQueryId();
			List tmpConfigList = parentPage.getQueryData(qId);
			resetQueryData(tmpConfigList);
			configList = tmpConfigList;
			queryIDText.setEditable(false);
		}
      return super.createDialogArea(parent);
	}

	/**
	 * sets the UI component values to the values in a List which has query information  
	 * @param tmpConfigList: List containing the query information 
	 */
	private void resetQueryData(List tmpConfigList){
		if (tmpConfigList!= null){
			queryIDText.setText((String)tmpConfigList.get(0));
			sqlStatementText.setText((String)tmpConfigList.get(1));
			rowNamespaceText.setText((String)tmpConfigList.get(2));
			groupByElementText.setText((String)tmpConfigList.get(3));
			rowNameText.setText((String)tmpConfigList.get(4));
			inputMapList = (List)tmpConfigList.get(5);
			outputMapList = (List)tmpConfigList.get(6);
			updateTable(tableInputMaps, inputMapList);
			updateTable(tableOutputMaps, outputMapList);
		} 	
	}
	/**
	 * Handler method for the Output Mapping
	 */
	private void handleNewOutputMapping() {
		//New Output Mapping
		AddOutputMappingDialog addOutputMappingDialog = new AddOutputMappingDialog(getShell(),null);
		addOutputMappingDialog.create();
		addOutputMappingDialog.getShell().setSize(500, 350);
		addOutputMappingDialog.open();
		if (addOutputMappingDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
			setMappingData(outputMapList, -1, addOutputMappingDialog.getMappingData());
			updateTable(tableOutputMaps,outputMapList);
		}
	}

	/**
	 * Handler method for the Output Mapping
	 */
	private void handleNewInputMapping() {
		//New Input Mapping
		AddInputParamDialog addInputParamDialog = new AddInputParamDialog(getShell(),null);
		addInputParamDialog.create();
		addInputParamDialog.getShell().setSize(500, 350);
		addInputParamDialog.open();
		if (addInputParamDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
			setMappingData(inputMapList, -1, addInputParamDialog.getMappingData());
			updateTable(tableInputMaps,inputMapList);
		}
	}

	/**
	 * Handle the Cancel button event 
	 */
	protected void cancelPressed() {
		super.cancelPressed();
		//Remove the DS Config from the 
	}

	/**
	 * Handle the OK button pressed
	 */
	protected void okPressed() {
		if (isAllDataValid()){
			super.okPressed();
			//Update the DS Configuration
			configChanged();
		}
	}
	
	private void showMsg(String msg){
		MessageBox messageBox  = new MessageBox(getShell(),SWT.OK);
		 messageBox.setMessage(msg);
		 messageBox.open();
	}
	/**
	 * Handle the configuration changes
	 */
	private void configChanged(){
		//update config with the change
		try {
			parentPage.setQueryData(
					(String)configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString()
						,configList);
			parentPage.configChanged();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * This method updates the List form current RDBMS Configuration.
	 * @return RDBMS COnfiguration 
	 */
	private void createListFromQuerySConfig(){
		configList.set(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID,queryIDText.getText());
		configList.set(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERY,sqlStatementText.getText());
		configList.set(WSO2DataserviceWizardStepTwo.SQLQUERY_NAME_SPACE,rowNamespaceText.getText());
		configList.set(WSO2DataserviceWizardStepTwo.SQLQUERY_ELEMENT,groupByElementText.getText());
		configList.set(WSO2DataserviceWizardStepTwo.SQLQUERY_ROW_NAME,rowNameText.getText());
		if (isQueryIDAlreadyPresent(
				configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString())){
			inputMappingButton.setEnabled(isEditMode);
			if (isEditMode) labelMessage.setText(""); 
			else labelMessage.setText("Query ID already exists.");
			outMappingButton.setEnabled(isEditMode && (!configList.get
					(WSO2DataserviceWizardStepTwo.SQLQUERY_ELEMENT).toString().equals("")));
		}else{
			labelMessage.setText("");
			if (configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString().equals(""))
			{
				inputMappingButton.setEnabled(false);
				outMappingButton.setEnabled(false);
			}else{
				inputMappingButton.setEnabled(true);
				outMappingButton.setEnabled(!configList.get
						(WSO2DataserviceWizardStepTwo.SQLQUERY_ELEMENT).toString().equals(""));
			}
		}
		
	}
	
	/**
	 * checks whether the given query id is already present in the query list 
	 * @param qid: query id tested
	 * @return true if the id is already present
	 */
	private boolean isQueryIDAlreadyPresent(String queryId){
		if (queryId.equals("")) return false;
		if (tmpQIDList == null) tmpQIDList = parentPage.getQueryIdList();
		return (tmpQIDList.indexOf(queryId) >= 0);
	}
	/**
	 * Return the current Config List
	 * @return
	 */
	public List getQueryList(){
		return configList;
	}

	/**
	 * validate the data entered
	 * @return true if all the data are valid
	 */
	private boolean isAllDataValid(){
		String validateString;
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString();
		if ((!isEditMode)&& (parentPage.getQueryData(validateString)!= null)){
			showMsg("Query Id '"+validateString+"' already present.");
			return false;
		}
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString().trim();
		if (validateString.equals("")){
			showMsg("Query id is required.");
			return false;
		}
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERY).toString();
		if (validateString.trim().equals("")){
			showMsg("Sql statement is required.");
			return false;
		}
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_ELEMENT).toString();
		if (validateString.trim().equals("")){
			showMsg("Group by element is required.");
			return false;
		}else{
			if (!WSO2DataserviceWizardStepTwo.isStringValidXmlTag(validateString)){
				showMsg("Group by element string '"+validateString+"' is not a valid xml tag string.");
				return false;
			}
		}
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_ROW_NAME).toString();
		if (validateString.trim().equals("")){
			showMsg("Row name is required.");
			return false;
		}else{
			if (!WSO2DataserviceWizardStepTwo.isStringValidXmlTag(validateString)){
				showMsg("Row name string '"+validateString+"' is not a valid xml tag string.");
				return false;
			}
		}
		
		validateString = configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_NAME_SPACE).toString();
		if (!validateString.trim().equals("")){
			if (!WSO2DataserviceWizardStepTwo.isStringValidXmlTag(validateString)){
				showMsg("Row namespace string '"+validateString+"' is not a valid xml tag string.");
				return false;
			}
		}
		return true;
	}
	
	private void declareColumn(Table table, int width,String colName){
		TableColumn column = new TableColumn(table,SWT.NONE);
		column.setWidth(width);
		column.setText(colName);
	}
	
	/**
	 * Handle when an item in the query table is selected 
	 */
	private void handleInputMapTableItemSelected(){
		editButtonInputMap.setEnabled(tableInputMaps.getSelectionIndex()!= -1);
		deleteButtonInputMap.setEnabled(tableInputMaps.getSelectionIndex()!= -1);
	}
	
	/**
	 * Handle when an item in the query table is selected 
	 */
	private void handleOutputMapTableItemSelected(){
		editButtonOutputMap.setEnabled(tableOutputMaps.getSelectionIndex() != -1);
		deleteButtonOutputMap.setEnabled(tableOutputMaps.getSelectionIndex() != -1);
	}
	
	/**
	 * updates the input/output mapping tables with the mapping list data 
	 * @param tbl: table to update
	 * @param fillItems: the list mapping data to be updated from 
	 */
	private void updateTable(Table tbl,List fillItems) {
		if (fillItems !=  null) {
			int queryCount = fillItems.size();
			int indexSelected = tbl.getSelectionIndex();
			tbl.removeAll();
			if (queryCount > 0) {
				TableItem[] items = new TableItem[queryCount]; // An item for each field
				for (int i = 0; i < queryCount; i++) {
					items[i] = new TableItem(tbl, SWT.NONE);
					//Get the Query ID
					List iMapItem = (List)fillItems.get(i);
					if (tbl == tableInputMaps){
						items[i].setText(new String[]{
							iMapItem.get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString()
							,iMapItem.get(
									WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_SQL_TYPE).toString()
									});
					}else{
						items[i].setText(new String[]{
							iMapItem.get(WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_TYPE).toString()
							,iMapItem.get(
									WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_OUT_FIELD).toString()
							,iMapItem.get(
									WSO2DataserviceWizardStepTwo.SQLQUERY_OMAP_SQL_COL).toString()
												});
						
					}
				}
				if (indexSelected > (queryCount-1)){
					indexSelected = queryCount-1;
				}
				tbl.setSelection(indexSelected);
				//updateStatus(null);
			}
			tbl.redraw();
		}
		setTableVisibleStatus(tbl, fillItems.size() > 0);
	}
	
	/**
	 * Handle edit Query
	 */
	private void handleInputMapEditQuery() {
		if (tableInputMaps.getSelectionIndex() >= 0){
			AddInputParamDialog addInputParamDialog = new AddInputParamDialog(getShell(),
					(List)inputMapList.get(tableInputMaps.getSelectionIndex()));
			addInputParamDialog.create();
			addInputParamDialog.getShell().setSize(500, 350);
			addInputParamDialog.open();
			if (addInputParamDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
				updateTable(tableInputMaps,inputMapList);
			}
		}
	}

	/**
	 * Handle edit Query
	 */
	private void handleOutputMapEditQuery() {
		if (tableOutputMaps.getSelectionIndex() >= 0){
			AddOutputMappingDialog addOutputMappingDialog = new AddOutputMappingDialog(getShell(),
					(List)outputMapList.get(tableOutputMaps.getSelectionIndex()));
			addOutputMappingDialog.create();
			addOutputMappingDialog.getShell().setSize(500, 350);
			addOutputMappingDialog.open();
			if (addOutputMappingDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
				updateTable(tableOutputMaps,outputMapList);
			}
		}
	}
	
	/**
	 * Handle delete Query
	 */
	private void handleInputMapDeleteQuery() {
		if (tableInputMaps.getSelectionIndex() >= 0){
			MessageBox messageBox  = new MessageBox(getShell(),SWT.YES|SWT.NO);
			String queryId=configList.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString();
			String mappingName=((List)inputMapList.get(tableInputMaps.getSelectionIndex())).get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString();
			boolean isMappingUsedElsewhere=isInputMappingUsedinOperations(queryId, mappingName);
			if (isMappingUsedElsewhere){
				messageBox.setMessage("The selected input mapping is used in a query operation defined in the next page. Are you sure you want to remove the input mapping?");		
			}else{
				messageBox.setMessage("Are you sure you want to remove the selected input mapping?");
			}
			if (messageBox.open() == SWT.YES){
				if (isMappingUsedElsewhere) removeMappingDetailsFromOperationId(queryId, mappingName);
				removeQueryData(inputMapList, tableInputMaps.getSelectionIndex());
				updateTable(tableInputMaps,inputMapList);
				handleInputMapTableItemSelected();
			}
		}
	}
	
	/**
	 * Handle delete Query
	 */
	private void handleOutputMapDeleteQuery() {
		if (tableOutputMaps.getSelectionIndex() >= 0){
			MessageBox messageBox  = new MessageBox(getShell(),SWT.YES|SWT.NO);
			messageBox.setMessage("Are you sure you want to remove the selected output mapping?");
			if (messageBox.open() == SWT.YES){
				removeQueryData(outputMapList, tableOutputMaps.getSelectionIndex());
				updateTable(tableOutputMaps,outputMapList);
				handleOutputMapTableItemSelected();
			}
		}
	}
	
	/**
	 * change the visibility of a table
	 * @param tbl: table object
	 * @param visible
	 */
	private void setTableVisibleStatus(Table tbl,boolean visible){
		visible = true;
		tbl.setVisible(visible);
		if (visible){
			((GridData)tbl.getLayoutData()).heightHint = 80;
		}else{
			((GridData)tbl.getLayoutData()).heightHint = 0;
		}
	}
	
	/**
	 * update the input/output mapping data to the input/output mapping list
	 * @param mapList: input/output maplist to update
	 * @param index: the index of the existing input/output mapping. -1 for a new mapping
	 * @param data: the data to be updated for the mapping
	 * @return the index of mapping data updated in the list.
	 */
	public int setMappingData(List mapList,int index, List data){
		if ((index < 0) || (index > (mapList.size()-1))){
			mapList.add(data);
			index = mapList.size()-1;
		}else{
			mapList.set(index, data);
		}
		return index;
	}
	
	/**
	 * remove the mapping data from the input/output list
	 * @param mapList
	 * @param index: index of the mapping in the list to remove
	 */
	public void removeQueryData(List mapList,int index){
		if ((index >= 0) && (index < mapList.size())){
			mapList.remove(index);
		}
	}
	

	public boolean isInputMappingUsedinOperations(String queryId,String mappingName){
		PersistentDSContext dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		List operationToUIMapping = dsContext.getOperationData();
		if (operationToUIMapping == null){
			return false;
		}else{
			List opIds=WSO2DataserviceWizardStepThree.getOperationIdList(operationToUIMapping);
			for (int i = 0; i < opIds.size(); i++) {
				List opData=WSO2DataserviceWizardStepThree.getOperationData(operationToUIMapping,opIds.get(i).toString());
				if (opData.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString().equals(queryId)){
					HashMap mappings=(HashMap)opData.get(WSO2DataserviceWizardStepThree.SQLOP_VALUE_MAPS);
					if (mappings.containsKey(mappingName)){
						//if (!mappings.get(mappingName).toString().equals("")){
						return true;
						//}
					}
				}
				
			}
		}
		return false;
	} 
	
	public void removeMappingDetailsFromOperationId(String queryId,String mappingName){
		PersistentDSContext dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		List operationToUIMapping = dsContext.getOperationData();
		if (operationToUIMapping == null){
			return;
		}else{
			List opIds=WSO2DataserviceWizardStepThree.getOperationIdList(operationToUIMapping);
			for (int i = 0; i < opIds.size(); i++) {
				List opData=WSO2DataserviceWizardStepThree.getOperationData(operationToUIMapping,opIds.get(i).toString());
				if (opData.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString().equals(queryId)){
					HashMap mappings=(HashMap)opData.get(WSO2DataserviceWizardStepThree.SQLOP_VALUE_MAPS);
					if (mappings.containsKey(mappingName)){
						mappings.remove(mappingName);
					}
				}
				
			}
		}
	}
	
}