/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepOne;

public class ExelConfigurationDialog extends Dialog{

	private Text exelFilePath;
	private Button browseButton;
	private PersistentDSContext dsContext;
	private WSO2DataserviceWizardStepOne parentPage;
	private List configList;
	
	public ExelConfigurationDialog(Shell parentShell, WSO2DataserviceWizardStepOne page) {
		super(parentShell);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		parentPage = page;
	}

	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 8;
		layout.verticalSpacing = 10;
		
		Label label = new Label(container, SWT.NULL);
		label.setText("Configuration \nExel File Location :");
		GridData gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		exelFilePath = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 6;
		exelFilePath.setLayoutData(gd);
		exelFilePath.setText("path-to-your-excel-file");
		exelFilePath.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		browseButton = new Button(container, SWT.BORDER);
		browseButton.setText("Browse");
		gd = new GridData();
		gd.horizontalSpan = 1;
		browseButton.setLayoutData(gd);
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleEdit(parent.getShell());
			}
		});
		
		return super.createDialogArea(parent);
	}
	
	private void handleEdit(Shell parent) {
		FileDialog fileDialog = new FileDialog(parent);
		fileDialog.open();
		fileDialog.setText("Open");
        String[] filterExt = { "*.xls", "*.doc" };
        fileDialog.setFilterExtensions(filterExt);
        String selected = fileDialog.open();
		exelFilePath.setText(selected);
	}

	/**
	 * Handle the Cancel button event 
	 */
	protected void cancelPressed() {
		super.cancelPressed();
		//Remove the DS Config from the 
	}

	/**
	 * Handle the OK button pressed
	 */
	protected void okPressed() {
		super.okPressed();
		//Update the DS Configuration
		configChanged();
	}
	
	/**
	 * Handle the configuration changes
	 */
	private void configChanged(){
		//update config with the change
		try {
			OMElement root = DataServiceXMLUtil.getOMDocFromString(dsContext.getDSConfig());
			OMElement element =DataServiceXMLUtil.createConfigElement(configList);
			//Remove the existing data service config
			Iterator children = root.getChildElements();
			while(children.hasNext()){
				OMElement node = (OMElement)children.next();
			        if (node.getLocalName().equals("config")) {
						node.detach();
					}
			}
			root.addChild(element);
			OutputStream xmlOutputStream = new ByteArrayOutputStream();
			root.serialize(xmlOutputStream);
			//call refresh configuration
			OutputStream stream = DataServiceXMLUtil.prettyPrintDSConfig(xmlOutputStream.toString());
			parentPage.refreshConfig(stream.toString());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * This method updates the List form current RDBMS Configuration.
	 * @return RDBMS COnfiguration 
	 */
	private void createListFromRDBMSConfig(){
		configList = new ArrayList();
		configList.add(new String[]{"excel_datasource",exelFilePath.getText()});
	}

}


