/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepOne;

/**
 * Dialog represent the JNDI configuration UI
 */
public class JNDIConfigurationDialog extends Dialog{

	private Text jndiClassText;
	private Text providerURLText;
	private Text resourceNameText;
	private Text userNameText;
	private Text passwordText;
	private PersistentDSContext dsContext;
	private WSO2DataserviceWizardStepOne parentPage;
	private List configList;
	
	public JNDIConfigurationDialog(Shell parentShell, WSO2DataserviceWizardStepOne page) {
		super(parentShell);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		parentPage = page;
	}

	/**
	 * Create the dialog area with the controls
	 */
	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;
		
		Label label = new Label(container, SWT.NULL);
		label.setText("JNDI Data Source Configuration");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("JNDI Context Class *");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		jndiClassText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		jndiClassText.setLayoutData(gd);
		jndiClassText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Provider URL *");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		providerURLText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		providerURLText.setLayoutData(gd);
		providerURLText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Resource Name *");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		resourceNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		resourceNameText.setLayoutData(gd);
		resourceNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Username");
		gd = new GridData();
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		userNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		userNameText.setLayoutData(gd);
		userNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Password");
		gd = new GridData();;
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		passwordText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		passwordText.setLayoutData(gd);
		passwordText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		return super.createDialogArea(parent);
	}
	
	/**
	 * Handle the Cancel button event 
	 */
	protected void cancelPressed() {
		super.cancelPressed();
		//Remove the DS Config from the 
	}

	/**
	 * Handle the OK button pressed
	 */
	protected void okPressed() {
		super.okPressed();
		//Update the DS Configuration
		configChanged();
	}
	
	/**
	 * Handle the configuration changes
	 */
	private void configChanged(){
		//update config with the change
		try {
			OMElement root = DataServiceXMLUtil.getOMDocFromString(dsContext.getDSConfig());
			OMElement element =DataServiceXMLUtil.createConfigElement(configList);
			//Remove the existing data service config
			Iterator children = root.getChildElements();
			while(children.hasNext()){
				OMElement node = (OMElement)children.next();
			        if (node.getLocalName().equals("config")) {
						node.detach();
					}
			}
			root.addChild(element);
			OutputStream xmlOutputStream = new ByteArrayOutputStream();
			root.serialize(xmlOutputStream);
			//call refresh configuration
			OutputStream stream = DataServiceXMLUtil.prettyPrintDSConfig(xmlOutputStream.toString());
			parentPage.refreshConfig(stream.toString());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * This method updates the List form current RDBMS Configuration.
	 * @return RDBMS COnfiguration 
	 */
	private void createListFromRDBMSConfig(){
		configList = new ArrayList();
		configList.add(new String[]{"jndi_context_class",jndiClassText.getText()});
		configList.add(new String[]{"jndi_provider_url",providerURLText.getText()});
		configList.add(new String[]{"jndi_resource_name",resourceNameText.getText()});
		configList.add(new String[]{"jndi_username",userNameText.getText()});
		configList.add(new String[]{"jndi_password",passwordText.getText()});
	}
	
}
