/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.dialog;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;
import org.wso2.ws.dataservice.ide.wizard.WSO2DataserviceWizardStepOne;

public class RDBMSConfigurationDialog extends Dialog{

	private Combo databaseTypeCombo;
	private Button testConButton;
	private Text driverClassText;
	private Text jdbcURLText;
	private Text userNameText;
	private Text passwordText;
	private Text minPoolSizeText;
	private Text maxPoolSizeText;
	private Label connectionResultLabel;
	private PersistentDSContext dsContext;
	private WSO2DataserviceWizardStepOne parentPage;
	private List configList;
	
	public RDBMSConfigurationDialog(Shell parentShell, WSO2DataserviceWizardStepOne page) {
		super(parentShell);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		parentPage = page;
	}

	protected Control createDialogArea(final Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);

		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 10;
		
		Label label = new Label(container, SWT.NULL);
		label.setText("Database Type");
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		databaseTypeCombo = new Combo(container,SWT.NULL);
		loadDataSourceCombo();
		databaseTypeCombo.select(0);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		databaseTypeCombo.setLayoutData(gd);
		databaseTypeCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				comboChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Driver Class");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		driverClassText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		driverClassText.setLayoutData(gd);
		driverClassText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("JDBC URL");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		jdbcURLText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		jdbcURLText.setLayoutData(gd);
		jdbcURLText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("User Name");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		userNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		userNameText.setLayoutData(gd);
		userNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Password");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		passwordText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		passwordText.setLayoutData(gd);
		passwordText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		
		label = new Label(container, SWT.NULL);
		label.setText("Minimum Pool Size");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		minPoolSizeText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		minPoolSizeText.setLayoutData(gd);
		minPoolSizeText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Maximum Pool Size");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		label.setLayoutData(gd);
		
		maxPoolSizeText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		maxPoolSizeText.setLayoutData(gd);
		maxPoolSizeText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				createListFromRDBMSConfig();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		testConButton = new Button(container, SWT.NULL);
		testConButton.setText("Test Connection");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
		testConButton.setLayoutData(gd);
		testConButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleTestCon(parent.getShell());
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		connectionResultLabel = new Label(container, SWT.NULL);
		connectionResultLabel.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		connectionResultLabel.setLayoutData(gd);
		
		// Load the Default Database parameters for the user.
		loadDefaultDatabaseParams(databaseTypeCombo.getText());
		createListFromRDBMSConfig();
		return super.createDialogArea(parent);
	}
	
	private void handleTestCon(Shell parent) {
		//Test Connection Logic
		try { 
			//First Try to load the driver class
			Class.forName(driverClassText.getText());
			//Try to connect
			Connection dbConnection = DriverManager.getConnection(jdbcURLText.getText(),
					userNameText.getText(), passwordText.getText());
			if (dbConnection != null) {;
				// Update the result label
				updateConnectionResults("Test Connection Successful !!",null);
			}
		} catch(ClassNotFoundException e){ 
			updateConnectionResults("Unable to load the driver class!!",e);
		} 
		catch( SQLException e ){
			updateConnectionResults("Couldn't get connection!!", e);
		}
	}
	
	private void updateConnectionResults(String message, Exception e){
		connectionResultLabel.setText("Ping Result :" +
				message + "\n" + ((e==null)?"":e.getMessage()));
	}

	/**
	 * Add the content to the data source combo.
	 */
	private void loadDataSourceCombo() {
		databaseTypeCombo.add(WSO2DataserviceWizardConstant.databaseTypeMySQL);
		databaseTypeCombo.add(WSO2DataserviceWizardConstant.databaseTypeApacheDerby);
	}
	
	/**
	 * Load the Default Database parameters for the user.
	 * User can change them appropriately
	 */
	private void loadDefaultDatabaseParams(String databaseSelection) {
		if (WSO2DataserviceWizardConstant.databaseTypeMySQL.equals(databaseSelection)) {
			driverClassText.setText(WSO2DataserviceWizardConstant.defaultDriverClassMySQL);
			jdbcURLText.setText(WSO2DataserviceWizardConstant.defaultProtocolMySQL);
			
		}else if (WSO2DataserviceWizardConstant.databaseTypeApacheDerby.equals(databaseSelection)){
			driverClassText.setText(WSO2DataserviceWizardConstant.defaultDriverClassDerby);
			jdbcURLText.setText(WSO2DataserviceWizardConstant.defaultProtocolDerby);
		}else {
			// Do not load, keep empty
		}
	}
	
	/**
	 * Ensures that both text fields are set.
	 */
	private void comboChanged() {
		//Load the default database params when the user change the database type.
		loadDefaultDatabaseParams(databaseTypeCombo.getText());
		//validate the plug-in
	}

	/**
	 * Handle the Cancel button event 
	 */
	protected void cancelPressed() {
		super.cancelPressed();
		//Remove the DS Config from the 
	}

	/**
	 * Handle the OK button pressed
	 */
	protected void okPressed() {
		super.okPressed();
		//Update the DS Configuration
		configChanged();
	}
	
	/**
	 * Handle the configuration changes
	 */
	private void configChanged(){
		//update config with the change
		try {
			OMElement root = DataServiceXMLUtil.getOMDocFromString(dsContext.getDSConfig());
			OMElement element =DataServiceXMLUtil.createConfigElement(configList);
			//Remove the existing data service config
			Iterator children = root.getChildElements();
			while(children.hasNext()){
				OMElement node = (OMElement)children.next();
			        if (node.getLocalName().equals("config")) {
						node.detach();
					}
			}
			root.addChild(element);
			OutputStream xmlOutputStream = new ByteArrayOutputStream();
			root.serialize(xmlOutputStream);
			//call refresh configuration
			OutputStream stream = DataServiceXMLUtil.prettyPrintDSConfig(xmlOutputStream.toString());
			parentPage.refreshConfig(stream.toString());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * This method updates the List form current RDBMS Configuration.
	 * @return RDBMS COnfiguration 
	 */
	private void createListFromRDBMSConfig(){
		configList = new ArrayList();
		configList.add(new String[]{"org.wso2.ws.dataservice.driver",driverClassText.getText()});
		configList.add(new String[]{"org.wso2.ws.dataservice.protocol",jdbcURLText.getText()});
		configList.add(new String[]{"org.wso2.ws.dataservice.user",userNameText.getText()});
		configList.add(new String[]{"org.wso2.ws.dataservice.password",passwordText.getText()});
		configList.add(new String[]{"org.wso2.ws.dataservice.minpoolsize",minPoolSizeText.getText()});
		configList.add(new String[]{"org.wso2.ws.dataservice.maxpoolsize",maxPoolSizeText.getText()});
	}

}