/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.w3c.dom.Document;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

/**
 * Utility class for the Data Services wizard 
 */
public class DataServiceXMLUtil {
	
	/**
	 * Utility method for pretty print the raw string data service configuration 
	 * @param rawConfig : in String format
	 * @return the output stream 
	 * @throws Exception 
	 */
	public static OutputStream prettyPrintDSConfig(String rawConfig) throws Exception{
		
		Document doc = getXMLDocFromString(rawConfig);
		
		return prettyPrintDSConfig(doc);
	}
	
	/**
	 * Utility method for pretty print the raw string data service configuration 
	 * @param rawConfig : in as XML Document
	 * @return the output stream 
	 * @throws Exception 
	 */
	public static OutputStream prettyPrintDSConfig(Document doc) throws Exception{
		
		OutputStream xmlOutputStream = new ByteArrayOutputStream();
		OutputFormat format = new OutputFormat(doc);
		format.setLineWidth(65);
		format.setIndenting(true);
		format.setOmitXMLDeclaration(true);
		format.setIndent(2);
		XMLSerializer serializer = new XMLSerializer(xmlOutputStream,format);
		serializer.serialize(doc);

		return xmlOutputStream;
	}
	
	/**
	 * Utility method for obtain the Document from XML string
	 * @param stringDoc
	 * @return Document
	 */
	public static Document getXMLDocFromString(String stringDoc) throws Exception{
		
		ByteArrayInputStream xmlStream = new ByteArrayInputStream(stringDoc.getBytes("UTF-8"));
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
		Document doc = docBuilder.parse(xmlStream);
		
		return doc;
	}
	
	/**
	 * Utility method for obtain the document from xml string
	 * @param stringDoc
	 * @return Document
	 */
	public static OMElement getOMDocFromString(String stringDoc) throws Exception{
		//create the parser
		XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(
				new ByteArrayInputStream(stringDoc.getBytes()));
		//create the builder
		StAXOMBuilder builder = new StAXOMBuilder(parser);
		//get the root element (in this case the envelope)
		OMElement documentElement =  builder.getDocumentElement();
		return documentElement;
	}
	
	/**
	 * Build the Configuration Element from the property list.
	 * @param propertyList : List of properties of Data Service Connection Configuration.
	 * @return Configuration Element
	 */
	public static OMElement createConfigElement(List propertyList){
		//create a factory
		OMFactory factory = OMAbstractFactory.getOMFactory();
		//use the factory to create three elements
		OMElement root = factory.createOMElement("config",null);
		
		Iterator  iterator = propertyList.iterator(); 
		while  ( iterator.hasNext()  )   {  
			String[] currentProperty = (String[])iterator.next() ;
			OMElement currentElement = factory.createOMElement("property",null);
			currentElement.addAttribute("name",currentProperty[0], null);
			currentElement.setText(currentProperty[1]);
			root.addChild(currentElement);
		}  
		return root;
	}
	
}
