/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.wizard;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;

/**
 * This WSO2 Data services creation wizard.
 */
public class WSO2DataserviceWizard extends Wizard implements INewWizard {
	
	private WSO2DataserviceWizardStepOne stepOne;
	private WSO2DataserviceWizardStepTwo stepTwo;
	private WSO2DataserviceWizardStepThree stepThree;
	private ISelection selection;
	private PersistentDSContext dsContext;

	/**
	 * Constructor for WSO2DataserviceWizard.
	 */
	public WSO2DataserviceWizard() {
		super();
		setNeedsProgressMonitor(true);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
	}
	
	/**
	 * Adding the pages to the wizard.
	 */
	public void addPages() {
		stepOne = new WSO2DataserviceWizardStepOne(selection);
		addPage(stepOne);
		stepTwo = new WSO2DataserviceWizardStepTwo(selection);
		addPage(stepTwo);
		stepThree = new WSO2DataserviceWizardStepThree(selection);
		addPage(stepThree);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}
	
	/**
	 * The worker method for finishing the data service creation wizard
	 */
	private void doFinish(IProgressMonitor monitor) throws CoreException {
		//Get the carbon home
		String carbonHome = dsContext.getCarbonHome();
		//Flush the data service configuration service hosting environment
		File dsCongigFile = new File(carbonHome+File.separator+
				WSO2DataserviceWizardConstant.repository + File.separator + 
				WSO2DataserviceWizardConstant.dataservices + File.separator +
				dsContext.getServiceName() + WSO2DataserviceWizardConstant.dbsFileExtention);
		try {
			dsCongigFile.createNewFile();
			FileWriter out = new FileWriter(dsCongigFile);
			out.write(dsContext.getDSConfig());
			out.close();
			System.out.println("Deployment Result : Data Service Diployed Successfully !!");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
	
	/**
	 * Page calculates enableness of the Finish button on each wizard page
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish() {
		IWizardPage[] pages = getPages();
		WizardPage wizardPage = null;
		for (int i = 0; i < pages.length; i++) {
			wizardPage = (WizardPage) pages[i];
			if (!(wizardPage.isPageComplete()))
				return false;
		}
		return true;
	}
	
	/**
	 * Checks the availability of the next page
	 */
	public IWizardPage getNextPage(IWizardPage page) {
		AbstractWSO2DataserviceWizardPage pageout = 
			(AbstractWSO2DataserviceWizardPage) super.getNextPage(page);
		if (pageout != null) {
			pageout.loadPreviousConfig();
		}
		return pageout;
	}
	
}