/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.wizard;

import java.io.File;
import java.io.OutputStream;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.dialog.CSVConfigurationDialog;
import org.wso2.ws.dataservice.ide.dialog.ExelConfigurationDialog;
import org.wso2.ws.dataservice.ide.dialog.JNDIConfigurationDialog;
import org.wso2.ws.dataservice.ide.dialog.RDBMSConfigurationDialog;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;

/**
 * The Wizard page that involves the configuration of the data service
 */
public class WSO2DataserviceWizardStepOne extends AbstractWSO2DataserviceWizardPage {
	
	private Text carbonHomeText;
	private Text serviceNameText;
	private Combo dataSource;
	private StyledText previewConfig;
	private PersistentDSContext dsContext;

	/**
	 * Constructor for WSO2DataserviceWizardStepOnePage.
	 * @param pageName
	 */
	public WSO2DataserviceWizardStepOne(ISelection selection) {
		super("WSO2 Data service Wizard Step One : Data Service Configuration");
		setTitle("WSO2 Data service Wizard Step One : Data Service Configuration");
		setDescription(WSO2DataserviceWizardConstant.pageDiscription);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(final Composite parent) {
		
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		
		Label label = new Label(container, SWT.NULL);
		label.setText("Carbon Home *");

		carbonHomeText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		carbonHomeText.setLayoutData(gd);
		loadDefaultCarbonHome();
		carbonHomeText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				carbonHomeChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse(parent.getShell());
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("Service Name :");

		serviceNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		serviceNameText.setLayoutData(gd);
		serviceNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				serviceNameTextChanged();
			}
		});
		
		
		label = new Label(container, SWT.NULL);
		label.setText("Data Source :");

		dataSource = new Combo(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		dataSource.setLayoutData(gd);
		loadDataSourceCombo();
		dataSource.select(0);
		dataSource.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				comboChanged();
			}
		});

		Button browseButton = new Button(container, SWT.PUSH);
		browseButton.setText(" Edit ");
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleEdit();
			}
		});
		
		label = new Label(container, SWT.HORIZONTAL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL|SWT.SEPARATOR);
		label.setLayoutData(gd);
		
		label = new Label(container, SWT.HORIZONTAL);
		label.setText("Preview of the Data Service Configuration");
		label.setLayoutData(gd);
		
		previewConfig = new StyledText(container,SWT.V_SCROLL | SWT.H_SCROLL);
		previewConfig.setText(loadPreviousConfig());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		gd.heightHint = 300;
		previewConfig.setLayoutData(gd);
		//previewConfig.scroll(100, 100, 100, 100, 100, 100, true);

		initialize();
		comboChanged();
		setControl(container);
		setPageComplete(false);
	}

	/**
	 * Load the default carbon home
	 */
	private void loadDefaultCarbonHome() {
		carbonHomeText.setText(dsContext.getCarbonHome());
	}

	/**
	 * Add the content to the data source combo.
	 */
	private void loadDataSourceCombo() {
		dataSource.add(WSO2DataserviceWizardConstant.dataSourceComboSelect);
		dataSource.add(WSO2DataserviceWizardConstant.dataSourceComboRDBMS);
		dataSource.add(WSO2DataserviceWizardConstant.dataSourceComboExel);
		dataSource.add(WSO2DataserviceWizardConstant.dataSourceComboCSV);
		dataSource.add(WSO2DataserviceWizardConstant.dataSourceComboJNDI);
	}
	
	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	private void initialize() {
		//Initialize the plug-in
	}
	
	/**
	 * Update the context with carbon home
	 */
	private void carbonHomeChanged() {
		if (validateCarbonHome(carbonHomeText.getText())) {
			dsContext.setCarbonHome(carbonHomeText.getText());
			updateStatus(null);
		} else {
			updateStatus("Carbon Home Validation Failed!, Please select a valid carbon home");
		}
	}
	
	/**
	 * This method will do needed validation of the carbon home
	 * These are the validation requirements
	 * 		Check carbon home exists
	 *		Check for the repository existence
	 *      Can create date service inside repository
	 * @param carhonHome
	 * @return boolean validation result
	 */
	private boolean validateCarbonHome(String carhonHome) {
		boolean validationResult = false;
		// check carbon home exists
		File carhonHomeFile = new File(carhonHome);
		if (!carhonHomeFile.exists()) {
			return validationResult;
		}
		// Check for the repository existence
		File carbonRepositoryLocationFile = new File(carhonHome
				+ File.separator + WSO2DataserviceWizardConstant.repository);
		if (!carbonRepositoryLocationFile.exists()) {
			return validationResult;
		}
		// Can create date service inside repository
		File carbonDSRepositoryLocation = new File(carbonRepositoryLocationFile.getPath() 
				+ File.separator + WSO2DataserviceWizardConstant.dataservices);
		if (!carbonDSRepositoryLocation.exists()) {
			carbonDSRepositoryLocation.mkdirs();
		}
		validationResult = true;
		return validationResult;
	}

	/**
	 * Handle the carbon home edit.
	 */
	private void handleBrowse(Shell parent) {
		DirectoryDialog dirDialog = new DirectoryDialog(parent);
		dirDialog.open();
		dirDialog.setText("Open");
		String selected = dirDialog.open();
		carbonHomeText.setText(selected);
	}

	/**
	 * Uses the standard container selection dialog to
	 * choose the new value for the container field.
	 */
	private void handleEdit() {
		if (dataSource.getSelectionIndex() == 0) {
			//Default Selection
		}else if (dataSource.getSelectionIndex() == 1){
			//RDBMS Selection
			//model = new RDBMSDSModel();
			RDBMSConfigurationDialog rdbmsConfigurationDialog 
					= new RDBMSConfigurationDialog(getShell(), this);
			rdbmsConfigurationDialog.create();
			rdbmsConfigurationDialog.getShell().setSize(550, 500);
			rdbmsConfigurationDialog.open();
		}else if (dataSource.getSelectionIndex() == 2){
			//Exel Selection
			ExelConfigurationDialog exelConfigurationDialog 
					= new ExelConfigurationDialog(getShell(), this);
			exelConfigurationDialog.create();
			exelConfigurationDialog.getShell().setSize(500, 150);
			exelConfigurationDialog.open();
		}else if (dataSource.getSelectionIndex() == 3){
			//CSV Selection
			CSVConfigurationDialog csvConfigurationDialog 
					= new CSVConfigurationDialog(getShell(), this);
			csvConfigurationDialog.create();
			csvConfigurationDialog.getShell().setSize(500, 400);
			csvConfigurationDialog.open();
		}else if (dataSource.getSelectionIndex() == 4){
			//JNDI Selection
			JNDIConfigurationDialog jndiConfigurationDialog 
					= new JNDIConfigurationDialog(getShell(), this);
			jndiConfigurationDialog.create();
			jndiConfigurationDialog.getShell().setSize(500, 400);
			jndiConfigurationDialog.open();
		}
	}
	
	private void serviceNameTextChanged(){
		//update config with the change
		try {
			Document dsConfig = DataServiceXMLUtil.getXMLDocFromString(dsContext.getDSConfig());
			Element dataElement = dsConfig.getDocumentElement();
			dataElement.setAttribute("name", serviceNameText.getText());
			//call refresh configuration
			OutputStream stream = DataServiceXMLUtil.prettyPrintDSConfig(dsConfig);
			refreshConfig(stream.toString());
			dsContext.setServiceName(serviceNameText.getText());
			updateStatus(null);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Handle the Combo changed operation
	 * Also need to handle the edit operation and status update
	 */
	private void comboChanged() {
		// Also need to handle service name edit operation 
		handleEdit();
		updateStatus(null);
	}

	/**
	 * Update the status of the Page depending on the values on items.
	 * @param message : message to display
	 */
	private void updateStatus(String message) {
		if (message == null) {
			if (!(carbonHomeText.getText().equals("") || carbonHomeText.getText().equals(
							WSO2DataserviceWizardConstant.defaultCarbonHOme))) {
				if ( !(serviceNameText.getText().equals(""))
						&& (dataSource.getSelectionIndex() != 0)) {
					setErrorMessage(message);
					setPageComplete(message == null);
				} else {
					setErrorMessage("Please select a Data Service Name and Configure the Data Source");
				}
			} else {
				setErrorMessage("Please select a valid Carbon Home");
			}
		}else {
			setErrorMessage(message);
		}
	}

	public String loadPreviousConfig(){
		String defaultConfig = null;
		try {
			defaultConfig = DataServiceXMLUtil.prettyPrintDSConfig(
					WSO2DataserviceWizardConstant.defaultDataServiceConfig).toString();
			refreshConfig(defaultConfig);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return defaultConfig;
	}
	
	public void refreshConfig(String config){
		//refresh the default configuration
		previewConfig.setText(config);
		//set the configuration on the eclipse preferences again
		dsContext.setDSConfig(config);
	}
	
}