/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.wizard;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.List;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.dialog.AddOperationDialog;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;

/**
 * The Wizard page that involves the configuration of Operations for DS
 */
public class WSO2DataserviceWizardStepThree extends AbstractWSO2DataserviceWizardPage {

	private int noOfLoadedOperations;
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private List operationToUIMapping;
	private StyledText previewConfig;
	private Table table;
	private Label label;
	private GridData gd;
	private PersistentDSContext dsContext;

	public static final int SQLOP_OPERATIONID = 0;
	public static final int SQLOP_QUERYID = 1;
	public static final int SQLOP_VALUE_MAPS = 2;

	/**
	 * Constructor for WSO2DataserviceWizardStepThree.
	 * @param pageName
	 */
	public WSO2DataserviceWizardStepThree(ISelection selection) {
		super("WSO2 Data service Wizard Step Three : Add Operation");
		setTitle("WSO2 Data service Wizard Step Three : Add Operation");
		setDescription(WSO2DataserviceWizardConstant.pageDiscription);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		
		final Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 7;
		layout.verticalSpacing = 10;

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		table = new Table(container,
				SWT.SINGLE|SWT.FULL_SELECTION|SWT.CHECK |SWT.V_SCROLL|SWT.H_SCROLL);
		table.setLinesVisible(true);
		table.setHeaderVisible(true); 
		table.setLayoutData(gd);
		declareColumn(table,500,"Current Operations");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		gd.heightHint = 100;
		table.setLayoutData(gd);
		table.setVisible(false);
		table.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				handleTableItemSelected();
			}
		});
		table.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e){
				handleEditOperation();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		newButton = new Button(container, SWT.PUSH);
		newButton.setText("Click to add New Operation");
		newButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleAddOperation();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		newButton.setLayoutData(gd);

		editButton = new Button(container, SWT.PUSH);
		editButton.setText("Edit Selected Operation");
		editButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleEditOperation();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		editButton.setLayoutData(gd);
		editButton.setEnabled(false);

		deleteButton= new Button(container, SWT.PUSH);
		deleteButton.setText("Delete Selected Operation");
		deleteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDeleteOperation();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		deleteButton.setLayoutData(gd);
		deleteButton.setEnabled(false);

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 4;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL);
		label.setText("Preview of the Data Service Configuration");
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		previewConfig = new StyledText(container,SWT.V_SCROLL | SWT.H_SCROLL);
		previewConfig.setText(loadPreviousConfig());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		gd.heightHint = 200;
		previewConfig.setLayoutData(gd);

		operationToUIMapping = new ArrayList();
		updateTable(operationToUIMapping);
		dsContext.setOperationData(operationToUIMapping);
		try{
			Class parameters[]={};
			dsContext.setMethodForUpdateOperationDataOnXml(this.getClass().getMethod("configChanged", parameters),this);
		} catch (NoSuchMethodException e) {
			e.printStackTrace(); 
        }
		initialize();
		setControl(container);
		setPageComplete(false);
	}

	/**
	 * Update the operation table with the new items
	 * @param fillItems : operation list to load to the table
	 */
	private void updateTable(List fillItems) {
		if (fillItems != null) {
			int operationCount = fillItems.size();
			String selectedId = null;
			int indexSelected = table.getSelectionIndex();
			if (table.getSelectionIndex() > -1){
				selectedId = table.getItem(table.getSelectionIndex()).getText();
			}
			table.removeAll();
			if (operationCount > 0) {
				table.removeAll();
				TableItem[] items = new TableItem[operationCount]; // An item for each field
				for (int i = 0; i < operationCount; i++) {
					items[i] = new TableItem(table, SWT.NONE);
					//Get the Operation ID
					List iOperationItem = (List)fillItems.get(i);
					items[i].setText(iOperationItem.get(0).toString());
					items[i].setChecked(true);//check them all by default
					if (selectedId != null){
						if (iOperationItem.get(SQLOP_OPERATIONID).toString().equals(selectedId)){
							indexSelected = i;
						}						
					}
				}
				if (indexSelected > (operationCount-1)){
					indexSelected = operationCount-1;
				}
				table.setSelection(indexSelected);
				handleTableItemSelected();
				updateStatus(null);
			}
		}
		table.setVisible(true);
	}


	private void declareColumn(Table table, int width,String colName){
		TableColumn column = new TableColumn(table,SWT.NONE);
		column.setWidth(width);
		column.setText(colName);
	}

	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	private void initialize() {
		//Initialize the plug-in
	}

	/**
	 * Handle add new operation
	 */
	private void handleAddOperation() {
		//Add Operation Selection
		AddOperationDialog addOperationDialog = new AddOperationDialog(getShell(), 
				WSO2DataserviceWizardConstant.addTag, this);
		addOperationDialog.create();
		addOperationDialog.getShell().setSize(500, 450);
		addOperationDialog.open();
		if (addOperationDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
			noOfLoadedOperations++;
			//operationToUIMapping.add(addOperationDialog.getOperationList());
			updateTable(operationToUIMapping);		
		}
	}
	
	/**
	 * Get the query data related to given query id
	 * @param operationId
	 * @return
	 */
	public static List getOperationData(List operationToUIMapping, String operationId){
		int queryCount = operationToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)operationToUIMapping.get(i);
			if (iQueryItem.get(SQLOP_OPERATIONID).toString().equals(operationId)){
				return iQueryItem;
			}						
		}
		return null;
	}

	/**
	 * Get the query data related to given query id
	 * @param operationId
	 * @return
	 */
	public List getOperationData(String operationId){
		return getOperationData(operationToUIMapping,operationId);
	}
	
	/**
	 * Update the query data in the query list for the relevant query id 
	 * @param operationId
	 * @param data
	 */
	public void setOperationData(String operationId, List data){
		int queryCount = operationToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)operationToUIMapping.get(i);
			if (iQueryItem.get(SQLOP_OPERATIONID).toString().equals(operationId)){
				operationToUIMapping.set(i, data);
				return;
			}						
		}
		operationToUIMapping.add(data);
	}
	
	/**
	 * Remove a query specified by query id
	 * @param operationID
	 */
	public static void removeOperationData(List operationToUIMapping, String operationID){
		int queryCount = operationToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)operationToUIMapping.get(i);
			if (iQueryItem.get(SQLOP_OPERATIONID).toString().equals(operationID)){
				operationToUIMapping.remove(i);
				return;
			}						
		}
	}
	
	/**
	 * Remove a query specified by query id
	 * @param operationID
	 */
	public void removeOperationData(String operationID){
		removeOperationData(operationToUIMapping,operationID);
	}
	
	/**
	 * Retrieve a list of query ids currently present
	 * @return
	 */
	public static List getOperationIdList(List operationToUIMapping){
		List IdList = new ArrayList();
		int queryCount = operationToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)operationToUIMapping.get(i);
			IdList.add(iQueryItem.get(SQLOP_OPERATIONID).toString());
		}
		return IdList;
	}
	
	/**
	 * Retrieve a list of query ids currently present
	 * @return
	 */
	public List getOperationIdList(){
		return getOperationIdList(operationToUIMapping);
	}
	
	/**
	 * return the query id of the selected item in the table
	 * @return
	 */
	public String getSelectedOperationId(){
		if (table.getSelectionIndex() > -1){
			return table.getItem(table.getSelectionIndex()).getText();
		}
		return null;
	}
	
	/**
	 * Handle when an item in the query table is selected 
	 */
	private void handleTableItemSelected(){
		editButton.setEnabled(table.getSelectionIndex() != -1);
		deleteButton.setEnabled(table.getSelectionIndex() != -1);
	}
	
	/**
	 * Handle edit operation
	 */
	private void handleEditOperation() {
		//Add Operation Selection
		if (table.getSelectionIndex() >= 0){
			AddOperationDialog addOperationDialog = new AddOperationDialog(getShell(), 
				WSO2DataserviceWizardConstant.editTag, this);
			addOperationDialog.create();
			addOperationDialog.getShell().setSize(500, 450);
			addOperationDialog.open();
		}
	}

	/**
	 * Handle delete operation
	 */
	private void handleDeleteOperation() {
		if (table.getSelectionIndex() >= 0){
			MessageBox messageBox  = new MessageBox(getShell(),SWT.YES|SWT.NO);
			messageBox.setMessage("Are you sure you want to remove the selected query operation?");
			if (messageBox.open() == SWT.YES){
				removeOperationData(getSelectedOperationId());
				noOfLoadedOperations--;
				updateTable(operationToUIMapping);
				configChanged();
				handleTableItemSelected();
			}
		}
	}


	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}
	
	public String loadPreviousConfig(){
		String defaultConfig = null;
		try {
			defaultConfig = DataServiceXMLUtil.prettyPrintDSConfig(dsContext.getDSConfig()).toString();
			refreshConfig(defaultConfig);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return defaultConfig;
	}

	public void refreshConfig(String config) {
		//refresh the default configuration
		previewConfig.setText(config);
		//set the configuration on the eclipse preferences again
		dsContext.setDSConfig(config);
	}

	/**
	 * Handle the configuration changes
	 */
	public void configChanged(){
		//update config with the change
		try {
			OMElement root = DataServiceXMLUtil.getOMDocFromString(dsContext.getDSConfig());
			//Remove the existing data service config
			Iterator children = root.getChildElements();
			while(children.hasNext()){
				OMElement node = (OMElement)children.next();
			        if (node.getLocalName().equals("operation")) {
						node.detach();
					}
			}
			List qIdList = getOperationIdList(dsContext.getOperationData());
			OMElement element;
			for(int i = 0;i < qIdList.size();i++){
				element = createQueryElement(getOperationData(dsContext.getOperationData(),(String)qIdList.get(i)));
				root.addChild(element);	
			}
			OutputStream xmlOutputStream = new ByteArrayOutputStream();
			root.serialize(xmlOutputStream);
			//call refresh configuration
			OutputStream stream = 
				DataServiceXMLUtil.prettyPrintDSConfig(xmlOutputStream.toString());
			refreshConfig(stream.toString());
			updateTable(operationToUIMapping);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Build the Query Element
	 * @param propertyList : List of properties of Data Service Connection Configuration.
	 * @return Query Element
	 */
	public OMElement createQueryElement(List configData){
		//create a factory
		OMFactory factory = OMAbstractFactory.getOMFactory();
		//use the factory to create three elements
		OMElement root = factory.createOMElement("operation",null);
		root.addAttribute("name",configData.get(SQLOP_OPERATIONID).toString(), null);

		OMElement currentElement = factory.createOMElement("call-query",null);

		currentElement.addAttribute("href",configData.get(SQLOP_QUERYID).toString(), null);
		List mappings=getQueryDataInputMappingList(configData.get(SQLOP_QUERYID).toString(), dsContext.getQueryData());
		for(int i = 0;i < mappings.size();i++){
			String name= ((List)mappings.get(i)).get(WSO2DataserviceWizardStepTwo.SQLQUERY_IMAP_NAME).toString();
			OMElement tcurrentElement = 
				factory.createOMElement("with-param",null);
			tcurrentElement.addAttribute("name",name, null);
			tcurrentElement.addAttribute("query-param",((HashMap)configData.get(SQLOP_VALUE_MAPS)).get(name).toString(), null);
			currentElement.addChild(tcurrentElement);
		}
		root.addChild(currentElement);
		
		
		root.addChild(currentElement);
		return root;
	}
	
	/**
	 * Get the query data related to given query id
	 * @param queryId
	 * @return
	 */
	public List getQueryDataInputMappingList(String queryId, List queryToUIMapping){
		int queryCount = queryToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)queryToUIMapping.get(i);
			if (iQueryItem.get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString().equals(queryId)){
				return (List)iQueryItem.get(WSO2DataserviceWizardStepTwo.SQLQUERY_INPUT_MAPS);
			}						
		}
		return null;
	}
}