/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.ws.dataservice.ide.wizard;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.events.*;

import org.wso2.ws.dataservice.ide.WSO2DataservicePlugin;
import org.wso2.ws.dataservice.ide.constant.WSO2DataserviceWizardConstant;
import org.wso2.ws.dataservice.ide.context.PersistentDSContext;
import org.wso2.ws.dataservice.ide.dialog.AddQueryDialog;
import org.wso2.ws.dataservice.ide.util.DataServiceXMLUtil;

/**
 * The Wizard page that involves the configuration of Queries for DS.
 */
public class WSO2DataserviceWizardStepTwo extends AbstractWSO2DataserviceWizardPage {

	private int noOfLoadedQuery;
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private List queryToUIMapping;
	private StyledText previewConfig;
	private Table table;
	private PersistentDSContext dsContext;

	public static final int SQLQUERY_QUERYID = 0;
	public static final int SQLQUERY_QUERY = 1;
	public static final int SQLQUERY_NAME_SPACE = 2;
	public static final int SQLQUERY_ELEMENT = 3;
	public static final int SQLQUERY_ROW_NAME = 4;
	public static final int SQLQUERY_INPUT_MAPS = 5;
	public static final int SQLQUERY_OUTPUT_MAPS = 6;
	
	public static final int SQLQUERY_IMAP_NAME = 0;
	public static final int SQLQUERY_IMAP_SQL_TYPE = 1;
	public static final int SQLQUERY_IMAP_INOUT_TYPE = 2;
	public static final int SQLQUERY_IMAP_ORDINAL = 3;
	
	public static final int SQLQUERY_OMAP_TYPE = 0;
	public static final int SQLQUERY_OMAP_OUT_FIELD = 1;
	public static final int SQLQUERY_OMAP_SQL_COL = 2;
	/**
	 * Constructor for WSO2DataserviceWizardStepTwo.
	 * @param pageName
	 */
	public WSO2DataserviceWizardStepTwo(ISelection selection) {
		super("WSO2 Data service Wizard Step Two : Add Query");
		setTitle("WSO2 Data service Wizard Step Two : Add Query");
		setDescription(WSO2DataserviceWizardConstant.pageDiscription);
		//Obtain the current context from eclipse preferences.
		dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		
		final Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 7;
		layout.verticalSpacing = 10;

		Label label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		table = new Table(container,
				SWT.SINGLE|SWT.FULL_SELECTION|SWT.CHECK|SWT.V_SCROLL|SWT.H_SCROLL);
		table.setLinesVisible(true);
		table.setHeaderVisible(true); 
		table.setLayoutData(gd);
		declareColumn(table,500,"Current Queries");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		gd.heightHint = 100;
		table.setLayoutData(gd);
		table.setVisible(false);
		table.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				handleTableItemSelected();
			}
		});
		table.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e){
				handleEditQuery();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		newButton = new Button(container, SWT.PUSH);
		newButton.setText("Click to add New Query");
		newButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleAddQuery();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		newButton.setLayoutData(gd);

		editButton = new Button(container, SWT.PUSH);
		editButton.setText("Edit Selected Query");
		editButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleEditQuery();
			}
		});
		editButton.setEnabled(false);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		editButton.setLayoutData(gd);

		deleteButton =  new Button(container, SWT.PUSH);
		deleteButton.setText("Delete Selected Query");
		deleteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDeleteQuery();
			}
		});
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		deleteButton.setLayoutData(gd);
		deleteButton.setEnabled(false);
		
		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 4;
		label.setLayoutData(gd);

		label = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		label = new Label(container, SWT.NULL);
		label.setText("");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);


		label = new Label(container, SWT.HORIZONTAL);
		label.setText("Preview of the Data Service Configuration");
		gd.horizontalSpan = 7;
		label.setLayoutData(gd);

		previewConfig = new StyledText(container,SWT.V_SCROLL | SWT.H_SCROLL);
		previewConfig.setText(loadPreviousConfig());
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 7;
		gd.heightHint = 200;
		previewConfig.setLayoutData(gd);

		queryToUIMapping = new ArrayList();
		dsContext.setQueryData(queryToUIMapping);
		updateTable(queryToUIMapping);
		initialize();
		setControl(container);
		setPageComplete(false);
	}


	/**
	 * Update the query table with the new items
	 * @param fillItems : query list to load to the table
	 */
	private void updateTable(List fillItems) {
		if (fillItems !=  null) {
			int queryCount = fillItems.size();
			String selectedId = null;
			int indexSelected = table.getSelectionIndex();
			if (table.getSelectionIndex() > -1){
				selectedId = table.getItem(table.getSelectionIndex()).getText();
			}
			table.removeAll();
			if (queryCount > 0) {
				TableItem[] items = new TableItem[queryCount]; // An item for each field
				for (int i = 0; i < queryCount; i++) {
					items[i] = new TableItem(table, SWT.NONE);
					//Get the Query ID
					List iQueryItem = (List)fillItems.get(i);
					items[i].setText(iQueryItem.get(SQLQUERY_QUERYID).toString());
					items[i].setChecked(true);//check them all by default
					if (selectedId != null){
						if (iQueryItem.get(SQLQUERY_QUERYID).toString().equals(selectedId)){
							indexSelected = i;
						}						
					}
					
				}
				if (indexSelected > (queryCount-1)){
					indexSelected = queryCount-1;
				}
				table.setSelection(indexSelected);
				updateStatus(null);
			}
			table.redraw();
		}
		table.setVisible(true);
	}

	/**
	 * Get the query data related to given query id
	 * @param queryId
	 * @return
	 */
	public List getQueryData(String queryId){
		int queryCount = queryToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)queryToUIMapping.get(i);
			if (iQueryItem.get(SQLQUERY_QUERYID).toString().equals(queryId)){
				return iQueryItem;
			}						
		}
		return null;
	}
	
	/**
	 * Update the query data in the query list for the relevant query id 
	 * @param queryId
	 * @param data
	 */
	public void setQueryData(String queryId, List data){
		int queryCount = queryToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)queryToUIMapping.get(i);
			if (iQueryItem.get(SQLQUERY_QUERYID).toString().equals(queryId)){
				queryToUIMapping.set(i, data);
				return;
			}						
		}
		queryToUIMapping.add(data);
	}
	
	/**
	 * Remove a query specified by query id
	 * @param queryId
	 */
	public void removeQueryData(String queryId){
		int queryCount = queryToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)queryToUIMapping.get(i);
			if (iQueryItem.get(SQLQUERY_QUERYID).toString().equals(queryId)){
				queryToUIMapping.remove(i);
				return;
			}						
		}
	}
	
	/**
	 * Retrieve a list of query ids currently present
	 * @return
	 */
	public static List getQueryIdList(List queryToUIMapping){
		List IdList = new ArrayList();
		int queryCount = queryToUIMapping.size();
		for (int i = 0; i < queryCount; i++) {
			List iQueryItem = (List)queryToUIMapping.get(i);
			IdList.add(iQueryItem.get(SQLQUERY_QUERYID).toString());
		}
		return IdList;
	}
	
	/**
	 * Retrieve a list of query ids currently present
	 * @return
	 */
	public List getQueryIdList(){
		return getQueryIdList(queryToUIMapping);
	}
	
	/**
	 * return the query id of the selected item in the table
	 * @return
	 */
	public String getSelectedQueryId(){
		if (table.getSelectionIndex() > -1){
			return table.getItem(table.getSelectionIndex()).getText();
		}
		return null;
	}
	
	private void declareColumn(Table table, int width,String colName){
		TableColumn column = new TableColumn(table,SWT.NONE);
		column.setWidth(width);
		column.setText(colName);
	}

	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	private void initialize() {
		//Initialize the plug-in
	}

	/**
	 * Handle when an item in the query table is selected 
	 */
	private void handleTableItemSelected(){
		editButton.setEnabled(table.getSelectionIndex() != -1);
		deleteButton.setEnabled(table.getSelectionIndex() != -1);
	}
	/**
	 * Handle add new Query
	 */
	private void handleAddQuery() {
		//Add Query Selection
		AddQueryDialog addQueryDialog = new AddQueryDialog(getShell(), 
				WSO2DataserviceWizardConstant.addTag, this);
		addQueryDialog.create();
		addQueryDialog.getShell().setSize(500, 800);
		addQueryDialog.open();
		if (addQueryDialog.getReturnCode() == org.eclipse.jface.window.Window.OK){
			noOfLoadedQuery++;
			updateTable(queryToUIMapping);			
		}
	}

	/**
	 * Handle edit Query
	 */
	private void handleEditQuery() {
		//Add Query Selection
		if (table.getSelectionIndex() >= 0){
			AddQueryDialog addQueryDialog = new AddQueryDialog(getShell(), 
				WSO2DataserviceWizardConstant.editTag, this);
			addQueryDialog.create();
			addQueryDialog.getShell().setSize(500, 800);
			addQueryDialog.open();
		}
	}

	/**
	 * Handle delete Query
	 */
	private void handleDeleteQuery() {
		if (table.getSelectionIndex() >= 0){
			MessageBox messageBox  = new MessageBox(getShell(),SWT.YES|SWT.NO);
			String queryId=((List)queryToUIMapping.get(table.getSelectionIndex())).get(WSO2DataserviceWizardStepTwo.SQLQUERY_QUERYID).toString();
			boolean isMappingUsedElsewhere=isQueryUsedinOperations(queryId);
			if (isMappingUsedElsewhere){
				messageBox.setMessage("The selected query is used in a query operation defined in the next page. Are you sure you want to remove this query?");		
			}else{
				messageBox.setMessage("Are you sure you want to remove the selected query?");
			}
			if (messageBox.open() == SWT.YES){
				if (isMappingUsedElsewhere) removeOperationFromOperationList(queryId);
				removeQueryData(getSelectedQueryId());
				noOfLoadedQuery--;
				updateTable(queryToUIMapping);
				configChanged();
				handleTableItemSelected();
			}
		}
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message  ==  null);
	}
	
	public String loadPreviousConfig(){
		String defaultConfig = null;
		try {
			defaultConfig = 
				DataServiceXMLUtil.prettyPrintDSConfig(dsContext.getDSConfig()).toString();
			refreshConfig(defaultConfig);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return defaultConfig;
	}

	public void refreshConfig(String config) {
		//refresh the default configuration
		previewConfig.setText(config);
		//set the configuration on the eclipse preferences again
		dsContext.setDSConfig(config);
	}

	/**
	 * Handle the configuration changes
	 */
	public void configChanged(){
		//update config with the change
		try {
			OMElement root = DataServiceXMLUtil.getOMDocFromString(dsContext.getDSConfig());
			//Remove the existing data service config
			Iterator children = root.getChildElements();
			while(children.hasNext()){
				OMElement node = (OMElement)children.next();
			        if (node.getLocalName().equals("query")) {
						node.detach();
					}
			}
			List qIdList = getQueryIdList(dsContext.getQueryData());
			OMElement element;
			for(int i = 0;i < qIdList.size();i++){
				element = createQueryElement(getQueryData((String)qIdList.get(i)));
				root.addChild(element);	
			}
			OutputStream xmlOutputStream = new ByteArrayOutputStream();
			root.serialize(xmlOutputStream);
			//call refresh configuration
			OutputStream stream = 
				DataServiceXMLUtil.prettyPrintDSConfig(xmlOutputStream.toString());
			refreshConfig(stream.toString());
			//update the operations section in the xml
			dsContext.updateOperationDataOnXml();
			refreshConfig(dsContext.getDSConfig());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Build the Query Element
	 * @param propertyList : List of properties of Data Service Connection Configuration.
	 * @return Query Element
	 */
	public OMElement createQueryElement(List configData){
		//create a factory
		OMFactory factory = OMAbstractFactory.getOMFactory();
		//use the factory to create three elements
		OMElement root = factory.createOMElement("query",null);
		root.addAttribute("id",configData.get(SQLQUERY_QUERYID).toString(), null);

		OMElement currentElement = factory.createOMElement("sql",null);
		currentElement.setText(configData.get(SQLQUERY_QUERY).toString());
		root.addChild(currentElement);
		
		for(int i = 0;i < ((List)configData.get(SQLQUERY_INPUT_MAPS)).size();i++){
			List mapData = (List)((List)configData.get(SQLQUERY_INPUT_MAPS)).get(i);
			currentElement = factory.createOMElement("param",null);
			currentElement.addAttribute(
					"name",mapData.get(SQLQUERY_IMAP_NAME).toString(), null);
			currentElement.addAttribute(
					"sqlType",mapData.get(SQLQUERY_IMAP_SQL_TYPE).toString(), null);
			currentElement.addAttribute(
					"type",mapData.get(SQLQUERY_IMAP_INOUT_TYPE).toString(), null);
			currentElement.addAttribute(
					"ordinal",mapData.get(SQLQUERY_IMAP_ORDINAL).toString(), null);
			root.addChild(currentElement);
		}
		
		currentElement = factory.createOMElement("result",null);
		if ((!configData.get(SQLQUERY_NAME_SPACE).toString().equals("")) && (((List)configData.get(SQLQUERY_OUTPUT_MAPS)).size() > 0)){
			currentElement.addAttribute("defaultNamespace",
				configData.get(SQLQUERY_NAME_SPACE).toString(), null);
		}
		currentElement.addAttribute("element",configData.get(SQLQUERY_ELEMENT).toString(), null);
		currentElement.addAttribute("rowName",configData.get(SQLQUERY_ROW_NAME).toString(), null);
		for(int i = 0;i < ((List)configData.get(SQLQUERY_OUTPUT_MAPS)).size();i++){
			List mapData = (List)((List)configData.get(SQLQUERY_OUTPUT_MAPS)).get(i);
			OMElement tcurrentElement = 
				factory.createOMElement(
						mapData.get(SQLQUERY_OMAP_TYPE).toString().toLowerCase(),null);
			tcurrentElement.addAttribute(
					"name",mapData.get(SQLQUERY_OMAP_OUT_FIELD).toString(), null);
			tcurrentElement.addAttribute(
					"column",mapData.get(SQLQUERY_OMAP_SQL_COL).toString(), null);
			currentElement.addChild(tcurrentElement);
		}
		root.addChild(currentElement);
		return root;
	}
	
	/**
	 * validates whether the string passed can be used as a valid string for a xml tag
	 * @param xmlTagString: the string that needs to be validated 
	 * @return true if the string is a valid xml tag string
	 */
	public static boolean isStringValidXmlTag(String xmlTagString){
		java.util.regex.Pattern p = java.util.regex.Pattern.compile("[A-Za-z]([A-Za-z0-9._]|-)*");
		return p.matcher(xmlTagString).matches();
	}
	
	public boolean isQueryUsedinOperations(String queryId){
		PersistentDSContext dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		List operationToUIMapping = dsContext.getOperationData();
		if (operationToUIMapping == null){
			return false;
		}else{
			List opIds=WSO2DataserviceWizardStepThree.getOperationIdList(operationToUIMapping);
			for (int i = 0; i < opIds.size(); i++) {
				List opData=WSO2DataserviceWizardStepThree.getOperationData(operationToUIMapping,opIds.get(i).toString());
				if (opData.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString().equals(queryId)){
					return true;
				}
				
			}
		}
		return false;
	} 
	
	public void removeOperationFromOperationList(String queryId){
		PersistentDSContext dsContext = WSO2DataservicePlugin.getDefault().getPersistentDSContext();
		List operationToUIMapping = dsContext.getOperationData();
		if (operationToUIMapping == null){
			return;
		}else{
			List opIds=WSO2DataserviceWizardStepThree.getOperationIdList(operationToUIMapping);
			for (int i = 0; i < opIds.size(); i++) {
				List opData=WSO2DataserviceWizardStepThree.getOperationData(operationToUIMapping,opIds.get(i).toString());
				if (opData.get(WSO2DataserviceWizardStepThree.SQLOP_QUERYID).toString().equals(queryId)){
					WSO2DataserviceWizardStepThree.removeOperationData(operationToUIMapping, opIds.get(i).toString());
				}
			}
		}
	}
}